<?php
/*
Copyright (C) 2011 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * 
 * 
 * MyMVC_config_ini 
 * 
 * Model pour traitement des données de config en base
 * remplace la lecture du fichier config.ini
 * 
 * 08/04/14 Optimisation du code
 * 
 *  
 * 
 */
class MyMVC_config_ini extends MyMVC_Model
{
   
   public function __construct() {
      $this->_sqlCreateTable = "( 
            `id_config` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `co_name` varchar(255) NOT NULL,
            `co_value` varchar(255) NOT NULL,
            `co_note` text,
            `co_system` VARCHAR( 1 ) NOT NULL DEFAULT  '0',
            PRIMARY KEY (`id_config`),
            UNIQUE (`co_name`)
          ) ENGINE=MyISAM DEFAULT CHARSET=utf8";

		parent::__construct('mymvc_config') ;
		$this->_setPrimaryKey("id_config",true) ;  // autoincremente	
   }
   
   public function beforeWrite($data) {
      if( $data['co_system']=='O' ) {
         unset($data['co_name']); // Protection contre la modification du nom de la variable
      } else {
         $data['co_system']='N'; // force la valeur
      }
      return $data;
   }
   
   public function afterRead($data,$aspect) { return $data; }
   
   /**
    * L'ajout est autorisé uniquement si le code 'addconfig' est spécifié 
    * 
    * @param type $col
    * @param type $data
    * @return string
    */
   public function isValid($col,$data) {
      $err= '';
      switch($col) {
         case 'co_name' : if($data['co_name']=='') $err = 'Veuillez saisir un nom '; break;
         case 'co_value' : if($data['co_value']=='') $err = 'Veuillez indiquer une valeur'; break;
      }
      return $err;
   }
       
   public function isDeletable($id) {
      $ret = true;
      $r = $this->_find($id);
      if( $r->co_system == 1) { // param system 
         $this->_setError('Variable système non supprimable');
         $ret = false;
      }
      return $ret;
   }
   
   /**
   * Liste les enregistrement stockées en base
   * 
   * @return array
   * 
   */
   public function getList() {
      $this->_setOrder("id_config");
      $liste = $this->_findAllArray();
      if( $liste != null ) {
         $this->response['data'] = $liste;
         $this->response['requestOk'] = true;
         $this->response['message'] = "OK";
      } else {
         $this->response['message'] = "Aucun enregistrement trouvé";
      }
      return $this->response;
   }
   
   /**
    * Enregistre en bdd
    * INSERT ou UPDATE
    * 
    * @return array
    * 
    */
   public function save() {
      if( $_POST['id_config']=='') {
         $n = $this->_create($_POST);
         $res = $n->_save() ;
      } else {
         $res = $this->_update($_POST['id_config'], $_POST);
      }
      if($res) {
         $this->response['requestOk'] = true;
      } else {
         $this->response['message'] = $n->_getErrors();
      }
       return $this->response;
   }
   
   /**
    * Lit un enregistrement
    * 
    * @param int $id
    * @return array
    * 
    */
   public function read($id) {
      $rec = $this->_find($id);
      if( $rec != null ) {
         $this->response['data'] = $rec->_toArray();
         $this->response['requestOk'] = true;
      } else {
         $this->response['message'] = 'Erreur enregistrement non trouvé';
      }
       return $this->response;
   }
   
   /**
    * Supprime un enregistrement en base
    * 
    * @param int $id
    * @return array
    */
   public function delete($id) {
      if( $this->_delete($id) ) {
          $this->response['requestOk'] = true;
      } else {
         $this->response['message'] = $this->_getLastError();
      }
       return $this->response;
   }
   
}

