<?php
/*
Copyright (C) 2005 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * 
 * 
 * MyMVC_datasource_ini
 * Model pour le traitement des datasources
 * remplace la lecture de datasources.ini
 * 
 * 08/04/14  Optimisation du code
 * 
 * 
 * 
 */

class MyMVC_datasource_ini extends MyMVC_Model
{
   
   public function __construct() {
      $this->_sqlCreateTable = "( 
            `id_datasource` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `ds_name` varchar(255) NOT NULL,
            `ds_context` varchar(255) NOT NULL,
            `ds_func` varchar(255) NOT NULL,
            `ds_cache` varchar(1) NOT NULL,
            `ds_note` text,
            PRIMARY KEY (`id_datasource`),
            UNIQUE (`ds_name`)
          ) ENGINE=MyISAM DEFAULT CHARSET=utf8";

		parent::__construct('mymvc_datasource') ;
		$this->_setPrimaryKey("id_datasource",true) ;  // autoincremente	
   }
   
   public function beforeWrite($data) {
      return $data;
   }
   
   public function afterRead($data,$aspect) { return $data; }
   
   public function isValid($col,$data) {
      $err= '';
      switch($col) {
         case 'ds_name' : if($data['ds_name']=='') $err = 'Veuillez saisir un nom de variable'; break;
         case 'ds_context' : if($data['ds_context']=='') $err = 'Veuillez indiquer le context (controler:action,..)'; break;
         case 'ds_func' : if($data['ds_func']=='') $err = 'Veuillez indiquer le nom de la fonction'; break;
      }
      return $err;
   }
       
   public function isDeletable($id) {
      //$this->_setError('not deletable');
      return true;
   }
    
   /**
   * Liste les enregistrement stockées en base
   * 
   * @return array
   * 
   */
   public function getList() {
      $this->_setOrder("ds_name");
      $liste = $this->_findAllArray();
      if( $liste != null ) {
         $this->response['data'] = $liste;
         $this->response['requestOk'] = true;
         $this->response['message'] = "OK";
      } else {
         $this->response['message'] = "Aucun enregistrement trouvé";
      }
      return $this->response;
   }
   
   /**
    * Enregistre en bdd
    * INSERT ou UPDATE
    * 
    * @return array
    * 
    */
   public function save() {
      if( $_POST['id_datasource']=='') {
         $n = $this->_create($_POST);
         $res = $n->_save() ;
      } else {
         $res = $this->_update($_POST['id_datasource'], $_POST);
      }
      if($res) {
         $this->response['requestOk'] = true;
      } else {
         $this->response['message'] = $n->_getErrors();
      }
       return $this->response;
   }
   
   /**
    * Lit un enregistrement
    * 
    * @param int $id
    * @return array
    * 
    */
   public function read($id) {
      $rec = $this->_find($id);
      if( $rec != null ) {
         $this->response['data'] = $rec->_toArray();
         $this->response['requestOk'] = true;
      } else {
         $this->response['message'] = 'Erreur enregistrement non trouvé';
      }
       return $this->response;
   }
   
   /**
    * Supprime un enregistrement en base
    * 
    * @param int $id
    * @return array
    */
   public function delete($id) {
      if( $this->_delete($id) ) {
          $this->response['requestOk'] = true;
      } else {
         $this->response['message'] = $this->_getLastError();
      }
       return $this->response;
   }
   
   
}

