<?php
/*
Copyright (C) 2009 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * 
 *  
 * Controler pour le controle des droits d'accès
 * Génère un tableau contenant pour chaque droit d'accès : 
 *    Groupe / Controler / Action / droit d'accès
 * 
 * l'appartemance au groupe est définit dans acl.ini avec les login users
 * 
 * Author : Auguste
 * Created : 29/09/09
 * 
 * Changelog :
 * 
 * 01/10/09 - loadRules(), checkAccess()
 * 02/10/09 - globalConfig->acls['groups'] and acls['rules]
 * 09/04/10 - groupe public => PUBLIC, séparateur action = : (au lieu de | )
 * 22/12/10 - ajout log info, redirect sur config->authUrl pour connexion
 * 11/10/12 - Trad FR
 * 29/07/13 - gestion des ACLs en base
 * 30/07/13 - ajout _userInGroup($g,$u)
 * 
 * ---
 * 
 * 
 * @TODO
 *    Utiliser noAuth=0|1 dans config.ini to disable auth process
 *    Modifier le lien entre groupes et users ??? * 
 *  
*/
  
class MyMVC_AccessControl
{  
	private $contexts ;        // contexts pour la gestion des droits 
	private $globalConfig ;    //  objet Config
	private $mymvc ;           // objet MyMVC
		
    function __construct() {
    		$this->mymvc 		= MyMVC::_getInstance() ;
    		$this->globalConfig = MyMVC_Config::_getInstance() ;
    }
    
    /**
     * Lecture de config/acl.ini pour créer : 
     *  
     * config['groups']
     * config['rules']
     *
     * @TODO 
     *   Sauver la config dans /cache/acl.mymvc
     * 
     */
    private function _loadAclIni()
    {
      $tmp = array();
      $acls = array();
    	if( $this->globalConfig->aclType == 'db' )
    	{
    		// recherche des données ACL dans la base
         $a = new Acl();
         $dbAcls = $a->_findAll('acl_enable="O" ' );
         foreach($dbAcls as $one) {
            switch( $one->acl_type) {
               case 'Groupe': {
                  $acls['groups'][$one->acl_name]=$one->acl_value;
                  break;
               }
               case 'Context':{
                  $acls['contexts'][$one->acl_name]=$one->acl_value;
                  break;
               }
               case 'Allow':{
                  $acls['allowRules'][$one->acl_name]=$one->acl_value;
                  break;
               }
               case 'Deny':{
                  $acls['denyRules'][$one->acl_name]=$one->acl_value;
                  break;
               }   
            } // switch
         } // foreach
    	} else {
         // aclType = 'file'
         if( is_file( BASE_PATH.'config/acl.ini') )
         {
            $acls = parse_ini_file( BASE_PATH.'config/acl.ini',true);
            // @TODO : Log 
         } else { 
            $this->mymvc->_log(MYMVC_ERROR,'config/acl.ini not found',MYMVC_LOG);
            throw new Exception('Error config/acl.ini not found');
         }
    	}
    	$tmp['groups'] = $acls['groups'] ;
		$this->contexts = $acls['contexts'] ;
		// @TODO Vérifier si count > 0
		$tmp['rules'] = array_merge( $this->_loadRules($acls['allowRules'],'allow') , $this->_loadRules($acls['denyRules'],'deny'));
      //d($tmp);
		$this->globalConfig->acls = $tmp;
		unset($tmp);	
		$this->mymvc->_log(MYMVC_INFO,'ACLs loaded',MYMVC_LOG);
    }
    
    /**
     * 
     * Extrait toutes les lignes d'ACL depuis $pRules
     * et découpe les lignes de context, pour créer des lignes uniques : 
     *   groupe/controler/action/droit
     * $pPolicy indique le bloc d'acl utilisé (allow ou deny) 
     *
     * @param array $pRules    
     * @param string $pPolicy
     * 
     */
    private function _loadRules($pRules,$pPolicy)
    {
    	$groupRules = array();
    	$acls = array();
    	foreach( $pRules as $group=>$rule)
    	{
    		$fullRules = array();
    		$aRules = explode(',',$rule);
    		foreach( $aRules as $oneRule)
    		{
    			if( strpos($oneRule,":") === FALSE ) // context ou controler:action ?
    			{
    				if( $this->contexts[$oneRule] != '' )  // context exist?
    				{
    					$c = explode(',',$this->contexts[$oneRule]) ;
    					$fullRules = array_merge( $fullRules, $c) ;
    				} else {
    					// @TODO log error context non trouvé
    				}
    			} else {
    				$fullRules[] = $oneRule ;
    			}
    		}
    		if( count($fullRules>0)) 
    		{
    			$groupRules[$group] = $fullRules ;
    		}	
    	}
    	
    	foreach($groupRules as $group=>$rules)
    	{
         foreach( $rules as $oneRule)
         {
            $ruleData = explode(':',$oneRule);	
            $acls[] = array('group'=>$group,'policy'=>$pPolicy,'controler'=>$ruleData[0],'action'=>$ruleData[1]); 
         }
		}
		return $acls;
    }
    
    /**
    * Controle si user connecté, sinon => renvoi sur page de login
    * si oui vérifie les droits d'accès par rapport à la ressource demandée (controler/action)
    * 
	 * @param string $pControler
	 * @param string $pAction
	 * @return bool
	 * 
	 */
	public function _checkAccess( $pControler, $pAction )
	{
       
		// Vérifie si les acl sont activées (config.ini)
		if($this->globalConfig->mymvcAcl === "0")
		{
			$this->mymvc->_log(MYMVC_INFO,'ACLs désactivées',MYMVC_LOG);
			return true;
		}
      
      $userId = 'anonymous';
		$this->_loadAclIni() ; 
		$accessRule = '' ; // droit d'accès 
      
		// controle si un accès public est autorisé
		foreach( $this->globalConfig->acls['rules'] as $oneRule)
		{
			if( $oneRule['group'] == 'PUBLIC'
			 		and ( $oneRule['controler']==$pControler or  $oneRule['controler'] == 'ALL')
			 		and ( $oneRule['action']==$pAction or $oneRule['action']=='ALL')
			 	 )
			{
			 	$accessRule = $oneRule ;
			 	break ; // fin traitement si une ACL correspond à la demande
			}
		}
		
      // Recherche les groupes de l'utilisateur
		if( $accessRule == '' )
		{
			if( $_SESSION['mymvc_userId'] == '' ) // pas de connnexion ?
			{
				$authUrl = BASE_URL . 'login.html' ;  // TODO utiliser une variable dans config.ini pour AuthURL
            if( $pControler != 'auth') {
               $this->mymvc->_log(MYMVC_INFO,'No access rule, redirect to auth controler : '. $authUrl , MYMVC_LOG);
               header('Location: ' .  $authUrl) ;  // renvoie sur page de login
            } else {
               MyMVC::_getInstance()->_showAppError('Aucun accès possible, veuillez vérifier vos ACLs');
            }
            exit();
			} else {
				$userId = $_SESSION['mymvc_userId'] ; 
				$userGroups = array('ALL'); // TODO store user group list in session['mymvc_userGroups] 					
				foreach( $this->globalConfig->acls['groups'] as $group=>$members) {
					$membersArray = explode( ',', $members) ;
					foreach( $membersArray as $one) {
						if( $one == $userId)  $userGroups[] = $group ;
					}
				}
			}
         //var_dump($this->globalConfig->acls['rules']);
			// Recherche d'une ACL pour le groupe de l'utilisateur
			foreach( $this->globalConfig->acls['rules'] as $oneRule)
			{
				if( in_array($oneRule['group'],$userGroups) // is user in acl group ? 
			 		and ( $oneRule['controler']==$pControler or  $oneRule['controler'] == 'ALL')
			 		and ( $oneRule['action']==$pAction or $oneRule['action']=='ALL')
			 	 )
				{
			 		$accessRule = $oneRule ;
			 		break ; // fin traitement si une ACL correspond à la demande
				}
			}
         // recherche s'il y a une exclusion de définit
         if( $accessRule != '' ) {
            foreach( $this->globalConfig->acls['rules'] as $oneRule) {
               if( in_array($oneRule['group'],$userGroups) // is user in acl group ? 
                  and $oneRule['controler']=="!$pControler" 
                  and ($oneRule['action']==$pAction OR $oneRule['action']=='ALL')
                 ) {
                 $accessRule = $oneRule ;
                 $accessRule['policy'] = 'deny' ; // annulation des droits d'accès 
                 break;
               }
            }
         }
		} 
	
      if( $accessRule == '') {
         $accessRule = array('policy'=>'default');
      }
     
		// Log info droit d'accès
		switch( $accessRule['policy'] ) {
			case 'allow' : {
				$message = "Access allowed  to $userId for $pControler:$pAction with rule: [". implode(' ',$accessRule)."]" ; 
				$allowAccess = true;
		 		break ;
			}
			case 'deny' : {
				$message = "Access denied to $userId for $pControler:$pAction with rule: [". implode(' ',$accessRule)."]" ;
				$allowAccess = false ; 
		 		break ;
			}
         case 'default':
			default : {
				$message = "Default policy  to $userId {$this->globalConfig->defaultAclPolicy} for $pControler:$pAction ";
				switch( $this->globalConfig->defaultAclPolicy)
				{
					case 'allow' : { $allowAccess = true ; break ; }
					case 'deny' : { $allowAccess = false ; break; }
					default : { $allowAccess = false ;  break ; } // acces refusé si pas de policy par défaut
				}
		 		break ;
			}				
		}
      //d($message);  
		$this->mymvc->_log(MYMVC_DEBUG,$message,MYMVC_LOG) ;
		return $allowAccess ;
	}
	
   /**
    * 
    * Check if a user is in a group
    * 
    * 
    * @param string $group
    * @param int/string $idUser
    * @return bool
    * 
    */
   public function _userInGroup($pGroup,$pIdUser)
   {
      $inGroup = false;
      if(count($this->globalConfig->acls['groups'])>0) {
         foreach( $this->globalConfig->acls['groups'] as $group=>$members) {
            if( $group == $pGroup) {
               $membersArray = explode( ',', $members) ;
               foreach( $membersArray as $one) {
                  if( $one == $pIdUser) $inGroup = true ;
               }
            }
         }
      }
      return $inGroup;
   }
}

