<?php
/*
Copyright (C) 2011 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

*/
/**
 * 
 * MyMVC_Config
 * 
 * 09/01/11 - ajout test DBCONFIG != ''
 * 28/02/11 - mise à jour mineurs
 * 09/03/12 - chargement configt DB depuis fichier config/db/xxx.ini, _loadDbConfig()
 * 11/10/12 - Define('LOG_DIR'...)
 * 30/01/13 - suppression lib amcharts
 * 15/03/13 - Transfert des coreLibs et Applibs dans mymvc.class.php
 * 08/04/14 - Ajout Traduction, optimisation code
 * 15/04/14 - Ajout BASE_URL pour les fichiers JS et CSS, pour autoriser l'ajout de fichier externe si besoin
 * 15/04/14 - Version 1.0, ajout version pour les classes principales
 * 15/04/14 - Insertion du code de config.inc.php pour limiter le nombre de fichier à charger
 * 23/04/14 - Ajout mymvc.css dans $mymvcCssFiles
 * 03/05/14 - MYMVC_SQL
 * 29/05/14 - Rempl exception par die() dans _loadDbConfig(), MyMVC n'est pas chargé et ne peut intercepter l'erreur
 * 
 * 1.0.4
 * 24/06/14 - Ajout nouveaux fichiers JS MyMVC_XXXXX
 * 01/07/14 - Ajout _myConfig dans MyMVC_Config pour remplacer les variables globales
 * 
 * 1.20
 * 
 * 
**/


define ('MyMVC_CONFIG_VERSION','1.20'); // 11/11/14
define('JQUERY_VERSION','2.1.1'); //2.0.3');

// log level
define("MYMVC_ERROR" , 4 );
define('MYMVC_WARNING' , 2 );
define('MYMVC_INFO', 1 );
define('MYMVC_DEBUG', 8 );
define('MYMVC_SQL',16); 

// Fichiers de logs, non  utilisé pour les logs en base
define('APP_LOG',1);   	// /applications/logs/app.log
define('MYMVC_LOG',2);  // /applications/logs/mymvc.log
//define('SQL_LOG',3);  	// /applications/logs/sql.log
define('ERROR_LOG',4);  // logs/error.log
                
// dossier des templates, relatif a /application
define('TEMPLATE_DIR',BASE_PATH.'templates/');

//define('TODAY_MYSQL', date('Y-m-d') ) ;
//define('TODAY_FR',date('d/m/Y'));

define('EXTERNAL_DIR','_externals/'); // sous dossier pour les templates externes
define('LAYOUT_DIR','_layouts/') ;  // sous dossier contenant les templates de layout
//define('TEMPLATE_EXT','html') ;  // extension des fichiers templates, utilisé pour générer le nom du template du action

## tag CSS et JS
define('CSS_TAG', '<link rel="stylesheet" type="text/css" href=":1" media="all" />' );
define('JS_TAG', '<script language="Javascript" type="text/javascript"  src=":1" ></script>' );

## header XML
define('HEADER_XML','Content-Type: text/xml') ;  // header xml pour les retours ajax
## prefix tables de la BDD
define('DB_PREFIX','');

# api jquery
//define('JQUERY','<script src="http://code.jquery.com/jquery-latest.js"></script>'); 
define('JQUERY','<script  language="Javascript" type="text/javascript" src="{BASE_URL}mymvc/jquery/jquery-'.JQUERY_VERSION.'.min.js"></script>');
//define('JQUERY','<script src="http://ajax.googleapis.com/ajax/libs/jquery/'.JQUERY_VERSION.'/jquery.min.js"></script>');


/*
// Fichier JS nécessaire pour MyMVC
$mymvcJsFilesOFF = array(
	'{BASE_URL}mymvc/js/MyMVC.js',
	'{BASE_URL}mymvc/js/MyMVC_Auth.js',
	'{BASE_URL}mymvc/js/MyMVC_Crud.js',
	'{BASE_URL}mymvc/js/MyMVC_Pagination.js',
	'{BASE_URL}mymvc/js/apis.js');
	//'{BASE_URL}mymvc/js/MyMVC_Wizard.js'); 

$mymvcCssFilesOFF = array(
		'{BASE_URL}mymvc/css/mymvc.css'
);
*/

session_start();

class MyMVC_Config implements ArrayAccess {
   
	private static $_instance = NULL;
	private $_configData = array() ;
	private $_myConfig = array();
	
	/* contructor */
	function __construct() {
		// set my configuration
		$this->_myConfig['urlToIgnore'] = array('favicon.ico','robots.txt','robot.txt');
		
		$this->_myConfig['cssFiles'] = array( '{BASE_URL}mymvc/css/mymvc.css' );
        
        $this->_configData['logEnabled'] = LOG_ENABLED;
        $this->_configData['logType'] = LOG_TYPE;
        $this->_configData['logLevel'] = LOG_LEVEL;
          
		if( file_exists( BASE_PATH.'config/config.ini') ) {
          $this->_configData = parse_ini_file( BASE_PATH.'config/config.ini', true);
          define('TEMPLATE_EXT',$this->_configData['templateExt']);
		} else {
          throw new Exception('MyMVC_config Fichier config.ini manquant') ;
		}
        $jsFiles = array(BASE_URL.'mymvc/js/MyMVC.js');
        if($this->_configData['js_MyMVC_Auth'] == 1) $jsFiles[] = '{BASE_URL}mymvc/js/MyMVC_Auth.js' ;
        if($this->_configData['js_MyMVC_Crud'] == 1) $jsFiles[] = '{BASE_URL}mymvc/js/MyMVC_Crud.js' ;
        if($this->_configData['js_MyMVC_Pagination'] == 1) $jsFiles[] = '{BASE_URL}mymvc/js/MyMVC_Pagination.js' ;
        if($this->_configData['js_MyMVC_Wizard'] == 1) $jsFiles[] = '{BASE_URL}mymvc/js/MyMVC_Wizard.js' ;
        $this->_myConfig['jsFiles'] = $jsFiles;
        
        if( MYMVC_DBCONFIG != '' ) {
          $dbFile = BASE_PATH.'config/db/'.MYMVC_DBCONFIG.'.ini' ;
          if( file_exists($dbFile) ) {
            $this->_configData['dbConfig'] = parse_ini_file($dbFile ,true);
          } else {
            display_html_error('Fichier de configuration base de données manquant '.MYMVC_DBCONFIG);
            die();
          }
		}
	}
  
	public function _getMyConfig($pName) {
		return $this->_myConfig[$pName];
	}
	
	public function _setMyConfig($pName,$pVal) {
		$this->_myConfig[$pName] = $pVal;
	}
	
	/**
	 * return instance
	 *
	 * @return MyMVC_Config()
	 */
	public static function _getInstance() {
	     if(self::$_instance == NULL ) {
	          self::$_instance = new MyMVC_Config();
	     }
	     return self::$_instance;
  	}
	
	/**
	 * Get a param value
	 *
	 * @param  string $pParam
	 * 
	 * @return  null or param value if found
	*/
	public function __get($pParam) {
		if( array_key_exists($pParam, $this->_configData) ) {
			return $this->_configData["$pParam"] ;
		} else {
			return null ;
		}
	}
	
	/**
	*  set a param 
	*  
	* @param string  $pParam 
	* @param string/array  $pVal
	* 
	* @return true
	*/
	public function __set($pParam, $pVal) {
		//throw new Exception('Erreur MyMVC_Config var are readonly');
		$this->_configData[$pParam] = $pVal ;
		return true ;
	}

	/**
	 * Functions to implements arrayAccess to class 
	*/	
	function offsetExists($pOffset) { }
	
	function offsetGet($pOffset) { return $this->$pOffset; }
	
	function offsetSet($pOffset,$pVal) { $this->set($pOffset,$pVal); }
	
	function offsetUnset($pOffset) { unset($this->_configData["$pOffset"]); }
	
}