<?php
/*
Copyright (C) 2005 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * 
 * 
 * Date :  		28/03/2007
 * Author : 	Auguste
 * 	 
 * main controler class, must be extended by all app controlers
 * 
 * 12/10/07 - ajout var $myModel
 * 14/10/07 - _getParam()
 * 31/01/09 - add controlerName param in __construct()
 * 
 * 20/05/09 - Version 1.0
 * 22/05/09 - replace APP_URL with BASE_URL, and APP_PATH with BASE_PATH
 * 20/07/09 - linkToPage()
 * 21/07/09 - add disableLog(), log()
 * 24/07/09 - enableLog(), setLogLevel() 
 * 14/08/09 - checkReferer()
 * 01/09/09 - checkReferer() only check if referer is in BASE_URL 
 * 11/09/09 - userAuth()
 * 08/10/09 - CRUD_ReadAction(), CRUd_SaveAction()
 * 07/11/09 - add _registry propertie, _save() registry in _redirect() method
 * 13/01/10 - suppr property  $action => $_action, _disableLog(), _enableLog()
 * 						_setLogLevel(), _linkTo(), _linkToPage()
 * 25/01/10 - _useDefaultView, _disableDefaultView();
 * 30/01/10 - _getParam(), wizardsaveAction()
 * 31/03/10 - _getAction(), _setViewName(), _getViewName(), _viewName
 * 20/04/10 - _getUrl()
 * 16/07/10 - ajout bloc try/catch sur CrudReadAction()
 * 04/04/11 - _disableDatasourceProcessing = false
 * 27/07/11 - _getParamsArray()
 * 12/10/12 - _init() 
 * 04/12/12 - _redirectToPage($url,$local)
 * 10/04/13 - _crudSaveAction()  maj pour utiliser _update() et non _create()
 * 22/07/13 - modif format data retour CrudReadAction() 
 * 27/11/13 - Mise à jour message d'erreur si appel méthode inexistante
 * 24/01/13 - _getTrad($texteId) et _setLocal($local)
 * 24/02/14 - suppression création registry dans le constructeur => __setRegistry()
 * 25/02/14 - ajout _addDebug()
 * 04/03/14 - suppr _getTrad()
 * 13/03/04 - Maj des trads
 * 08/04/14 - Optimisation Code
 * V 1.0.2
 * 16/05/14 - ajout id dans JSON retourné par crudSaveAction()
 * 06/06/14 - _redirectToPage()  ajout _save() de la registry
 * 1.0.3
 * 10/06/14 - Log nom des méthodes en mode DEBUG
 * 27/06/14 - crudListAction() 
 * 30/06/14 - Modif init Registry appel _getInstance() au lieu de New ..()
 * 10/09/14 - Utilisation de defaultLocal (config.ini) si param vide envoyé a _setLocal()
 * 20/10/14 - Ajout log sur crudDeleteAction(), crudReadAction() et crudSaveAction()
 * 
 * 
 * */
 
define ('MyMVC_CONTROLER_VERSION','1.0.3');  //  10/09/14

class MyMVC_Controler {   
  public $_params ;									// action params
  public $_view ;										// view object
  public $_action ;									// current action
	public $_name ;										// controler name
	public $_registry ;								// MyMVC_Registry object
	public $_useDefaultView ;					// 
	private $_viewName ;							// string
	public $_datasourceProcessing ;		// désactive la création des datasources 
	public $_viewVarProcessing ;
	
  public function __construct($pName=null) {
		$this->_name									= $pName ;
		$this->_view									= new MyMVC_View() ;
		$this->_useDefaultView				= true ;
		$this->_viewName							= '';
		$this->_datasourceProcessing	= true; 
		$this->_viewVarProcessing			= true; 
		$this->_registry							= MyMVC_Registry::_getInstance();
	}
	
   public function __call($func,$params) {
      throw New Exception(MyMVC::getTrad('mymvc_appelMethodeInexistanteDansControler').'['.$this->_name.']');
   }
   
   public function init(){}  // methode qui est appelé juste avant l'action en cours, permet de setter des variables de vue globales par exemple
   
   
  /** ajoute un texte a afficher dans la zone debug */
  public function _addDebug($pTexte) { MyMVC::_getInstance()->__addDebug($pTexte); }
  
	/** stockage de variable globale pendant l'exécution du script uniquement */
	public function _storeValue($pName,$pValue) { MyMVC::_getInstance()->_storeValue($pName,$pValue); }
	/** récupération de variable globale */
	public function _getValue($pName) { return MyMVC::_getInstance()->_getValue($pName); }
	
  /**
   * Set la lang (local) à utiliser
   * par défaut utilise la langue de config.ini
   * 
   * @param type $pLocal
  */
  public function _setLocal($pLocal) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_setLocal()',MYMVC_LOG);
		if( $pLocal=='') {
			$pLocal = MyMVC_Config::_getInstance()->defaultLocal ;
			//MyMVC::_getInstance()->_log(MYMVC_INFO,'No language to set! in _setLocal()');
		}
		MyMVC::_getInstance()->__loadLocal($pLocal);
  }
   
	/*
	public function _disableDefaultView()
	{
		$this->_useDefaultView = false ;
	}
	*/
	
  public function _setViewName($pName) { $this->_viewName = $pName; }
	
	public function _getViewName() { return $this->_viewName ; }
	
	/**
	 * enable all logs!
	 *
	 */
	public function _enableLog() {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_enableLog()',MYMVC_LOG);
		$cfg = MyMVC_Config::_getInstance() ;
		$cfg->logEnabled = 1 ;
	}
	
	/**
	 * set new Log Level
	 *
	 */
	public function _setLogLevel( $pNewLogLevel) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_setLogLevel()',MYMVC_LOG);
		$cfg = MyMVC_Config::_getInstance() ;
		$cfg->logLevel = $pNewLogLevel ;
	}
   
	/**
	 * disable all logs!
	 *
	 */
	public function _disableLog() {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_disableLog()',MYMVC_LOG);
		$cfg = MyMVC_Config::_getInstance() ;
		$cfg->logEnabled = 0 ;
	}
	
	/**
	 * log a message to appLog
	 *
	 * @param unknown_type $pMessage
	 */
	public function _log($pMessage) {
		if($pMessage!='')	{
			MyMVC::_getInstance()->_log(MYMVC_INFO,$pMessage); // app_log par défaut
		}
	}
	
	/**
	 * 10/10/07
    * Vérifie que le Referrer (page appelante)
	 * vient du même site
	 * 
	 * @param array|string $pAuthorizedReferer
	 * 
	 * @return bool
	 * 	
	 */
	public function _checkReferer($pAuthorizedReferer=null) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_checkReferer()',MYMVC_LOG);
  		if( ! (strpos( BASE_URL , $_SERVER['HTTP_REFERER']) >= 0) )	{
  			throw new Exception(MyMVC::getTrad('mymvc_pageNonAccessibleDepuisVotreAdresse'));
  		}
  	}
	
  /**
   * 01/04/07
   * Set current controler action
   * 
  */
  public function _setAction($pAction) { $this->_action = $pAction ; }
  
  /**
   * 31/03/10
   * @return string
   */
   public function _getAction() { return $this->_action  ; }
  
  
  /**
   * display controler/action view 
   * call all hooks methods , _beforeRender,_afterRender,_beforeShow and _afterShow
   * 
   * TODO   remove flash from here, cannot be global to MyMVC Framework
   * 
   */  
  public function _showView() {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_showView()',MYMVC_LOG);
   	$this->beforeRender($this->_view) ;
   	$this->_view->_render() ; // render layout + vue 
   	$this->afterRender($this->_view);  // 
      $this->_view->_sendHeaders(); // envoie de header http spéciaux si besoin
      print $this->_view->_getCode();
   	$this->afterShow($this->_view);
  } 
        
  /**  Hook  beforeRender  */
  public function beforeRender($pView) { } 
  
 	/** Hook afterRender */
  public function afterRender($pView) { }
  
  /** Hook afterShow */
  public function afterShow($pView) { }  
    
  /**
   * génére l'url pour un controleur et une action donnée
   * utilise le controleur en cours si non fournis
   * appel possible :
   * 
   * _linkTo( 'A1','C1')  controleur1/action1 
   * _linkTo( 'A3')   action3 du controleur en cours 
   * _linkTo()   controleur et action par défaut , def dans config
   * _linkTo('',C2)  controleur2 et action par défaut du controleur
   * 
   * un 3eme param permet de passer des param qui seront traité par le methode(action) du controleur
   * array('nom'=>val','nom2'=>'val2'....)
   * 
   */
  public function _linkTo($pAction=null,$pControler=null, $pParams=null) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_linkTo()',MYMVC_LOG);
  	$base = BASE_URL ;
  	if( $pControler ) {
   		$base .= $pControler ;
   		if( $pAction )	{
   			$base .= "/$pAction" ;
   		}
   	} else {
   		if( $pAction ) { // si juste action, on garde le controleur en cours
   			$base .= $this->_name."/$pAction" ;
   		}
   	}
   	if( $pParams !== null and  is_array($pParams) and count($pParams) > 0 )	{
         foreach($pParams as $n=>$v) {
            $v = urlencode($v);
            $n = urlencode($n);
            $base .= "/$n=$v" ;
         }
   	}
   	return $base ; 
  }

  /**
   * create an href link to a page on BASE_URL app
   *
   * @param unknown_type $pPageUrl
   * 
   * TODO: check if pageUrl exist in pages.ini
   */
  public function _linkToPage( $pPageUrl) { 
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_linkToPage()',MYMVC_LOG);
		return BASE_URL.$pPageUrl ;
	} 
    
  /**
   * retourne l'url complete pour $pAction
   * pour le controler courant
   *
   * @param string $pAction
   * @param array $pParams
   * 
   * @return string $url 
   * 
   * TODO ajouter test si $pAction existe
   * traitement de $pParams
   *  
   */
  public function _getUrl($pAction,$pParams=null) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_getUrl()',MYMVC_LOG);
  	$url  =  BASE_URL. $this->_name.'/'.$pAction ;
  	return $url;
  }
  
  /** 
   * 31/03/07
   * 
   * renvoie vers une autre page
   * appels possible => idem _linkTo
   * 
  */
  public function _redirectTo($pAction=null,$pControler=null,$pParams=null) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_redirectTo()',MYMVC_LOG);
		$url = '';
		if( $pControler ) {
			$url .= $pControler ;
			if( $pAction ) $url .= "/$pAction" ;
		} else {
         // si juste action, on garde le controleur en cours
			if( $pAction ) $url .= $this->_name."/$pAction" ;
		}
		$url = BASE_URL.$url ;
		if( $pParams ) {
         foreach($pParams as $n=>$v) {
            $v = urlencode($v);
            $n = urlencode($n);
            $url .= "/$n=$v" ;
         }
      }
      $this->_registry->_save(); // saved registry
      MyMVC::_getInstance()->_log(MYMVC_INFO,"Redirect to $url",MYMVC_LOG);
      header('Location: '.$url);
      exit;
  }	
  
  /**
   * Redirige vers une url locale ou distante (absolue)
   * 
   * @param string $pUrl
   * @param boolean $pLocal
   * 
	 * @TODO save registry before redirect
	 * 
   */
  public function _redirectToPage($pUrl,$pLocal=true) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::_redirectToPage()',MYMVC_LOG);
     if( $pUrl != '') {
			  $this->_registry->_save();
        if( $pLocal) $pUrl = BASE_URL.$pUrl;
        header('Location: '.$pUrl);
        exit;
     }
  }
  
  /**
   * CRUD read action 
   * read a redcord in DB, 
   * nead id and model from $_POST vars
   * to know with model to use and wich key to look for 
   * 
   * @return associative array   { requestOk => boolean, message=> string , data=> array or null } 
   * 
   */
	public function crudReadAction() {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::crudReadAction()',MYMVC_LOG);
		$this->_checkReferer();
		$response = array( 
					'requestOk'=>false,
					'message'=>MyMVC::getTrad('mymvc_CrudReadIdInvalide'),
					'data'=>'' ) ;
		
		$id =  $this->_params['id'];
		$model = $this->_params['crud_model'] ;
		if( $model == '' or $id == '' ) {
			$response['message'] = MyMVC::getTrad('mymvc_CrudReadDonneeInsuffisante');
		} else {
			try {
				$m = new $model();
				$m->_setAspect('form'); // aspect pour les fonctions setAferRead
				$rec = $m->_find($id) ;
				if( $rec != null) { // found?
					$response =  array( 'requestOk'=>true,
										'message'=>'',
										//'data'=>array( $rec->_toArray()) 
                              'data'=>$rec->_toArray()
									) ;
					$this->_log('crudReadAction successfully, model = '.$m->_table. ', read id = '.$this->_params['id'],MYMVC_LOG);
				} else {
				$response =  array( 'requestOk'=>false,
									'message'=>MyMVC::getTrad('mymvc_aucuneDonneeTrouvee'),
									'data'=>null 
								) ;	
				}
			} catch( Exception $e) {
				$response['message'] = $e->getMessage();
			}
		}
		$this->_view->_sendJsonResponse($response);
		return false ; 
	}

	/**
	 * crudListAction   27/06/14
	 * 
	 * appel un méthode (filter) d'un modele 
	 * le résultat est retourné en ne prenant en compte que les lignes concernées par
	 * par firstRec et NbRec
	 * Le résultat est retourné en JSON
	 * 
	 * L'appel Ajax doit utiliser formulaire avec 
	 * 
	 * mymvc_model => nom du model a utiliser
	 * 
	 * Facultatif
	 * 
	 * mymvc_modelFunc => méthode du model qui effectuera la recherche, doit retourné un array (_findAllArray())
	 * 
	 * si mymvc_modelFunc = '' une requête automatique sera exécuté avec :
			* mymvc_firstRec => 1er enregistrement
			* mymvc_nbRec => nombre d'enregistrement à extraire
			* mymvc_order => tri de la requête
			* mymvc_filter => string qui sera utilisé pour le where de la requete
	 * ces params sont facultatif, sans param la requetes fera un _findAllArray() en utilisant la config par défaut du model
	 * 
	 * la fonction du modele (mymvc_modelFunc) sera appelée avec en paramètre tous les champs du formulaire
	 * il est donc possible d'utiliser autant de champ que nécessaire en plus des champs mymvc_xxxxx
	 * 
	 * 
	 */
	public function crudListAction() {
		$json = array('requestOk'=>false,'message'=>'','data'=>null);
		if(!empty($this->_params['mymvc_model'])) {
			$modelName = $this->_params['mymvc_model'];
			$m = new $modelName();
			if(!empty($this->_params['mymvc_modelFunc'])) { // methode du model qui doit effectuer la requete
				$funcName = $this->_params['mymvc_modelFunc'];
				$json['data'] = $m->$funcName($this->_params);
			} else {
				// req automatique 
				// limit et offset
				if(!empty($this->_params['mymvc_firstRec'])) { // 1er REC
					$m->_setOffset( intVal($this->_params['mymvc_firstRec']) );
				}
				if(!empty($this->_params['mymvc_nbRec'])) {
					$m->_setLimit( intval($this->_params['mymvc_nbRec']) );
				}
				// order
				if(!empty($this->_params['mymvc_order'])) {
					$m->_setOrder($this->_params['mymvc_order']);
				}
				// filtre
				if(!empty($this->_params['mymvc_filter'])) {
					$json['data'] = $m->_findAllArray($this->_params['mymvc_filter']);
				} else {
					$json['data'] = $m->_findAllArray();
				}
			}
			$json['requestOk'] = true;
		} else { // pas de mymvc_model
			$json['message'] = MyMVC::getTrad('mymvc_CrudListModelIncorrect');
		}
		$this->_view->_sendJsonResponse($json);
		return false ; 
	}
	
	/**
	 * CRUD save action
	 * get _POST datas to save in DB
	 * crud_model store model name 
	 *
	 * @return  array   { requestOk, message } 
	 */
	public function crudSaveAction()	{
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::crudSaveAction()',MYMVC_LOG);
		//$this->_checkReferer(); // check referer, if not ok, redirect to error page
		
		$result['requestOk'] = true ;
    $result['message'] = MyMVC::getTrad('mymvc_enregistrementEffectueAvecSucces') ;
		$result['id'] = '';
		
		$model =  $this->_params['crud_model'] ;
		if( $model == '' ) {
			$result['message'] = MyMVC::getTrad('mymvc_nomModelInvalidCrudSaveNePeutEtreExecute');
			$result['requestOk'] = false ;
		} else {
			// check form values
			$m = new $model() ;
         $rec = $m->_create($this->_params);
         if( ! $rec->_save() ) {
            $result['requestOk'] = false ;
            $result['message'] = $rec->_getErrors() ; 
         } else {
					 $k = $m->_key;
					 $result['id'] = $rec->$k ; // id créé
					 $this->_log('crudSaveAction successfully, model = '.$m->_table. ', saved id = '.$this->_params['id'],MYMVC_LOG);
				 }
         /*$pk = $this->_params[$m->_key]; // valeur primary key
         if( $pk == '' ) //save new
         {
            $rec = $m->_create($_POST);
            if( ! $rec->_save() )
            {
               $result['requestOk'] = false ;
               $result['message'] = $rec->_getErrors() ; 
            } 
         } else {
            // update
            if( !$m->_update($pk,$this->_params) ) // insert or update
            {
               // error processing form
               $result['requestOk'] = false ;
               $result['message'] = $m->_getLastError(); // $oneError.'<br/>' ;
            } 
         }*/
		}
		$this->_view->_sendJsonResponse($result);
		return false ; 
	}
	
	/** generic CRUD delete function */
	public function crudDeleteAction() {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Controler::crudDeleteAction()',MYMVC_LOG);
		$this->_checkReferer();

		// error processing form
		$result['requestOk'] = true ;
		$result['message'] = '';
		
		//$model = ucFirst($this->_getParam(('crud_model'))) ;
	  $model =  $this->_params['crud_model'] ;
		if( $model == '' ) {
			$result['message'] = MyMVC::getTrad('mymvc_crudDeleteModelInvalide');
			$result['requestOk'] = false ;
		} else {
			$m = new $model() ;
			if( !$m->_delete(  $this->_params['id'] ) ) {
				// error processing form
				$result['requestOk'] = false ;
				$result['message'] = $m->_getLastError() ;
			} else {
				$result['message'] = MYMVC::getTrad('mymvc_suppressionEffectueeAvecSucces') ;
				$this->_log('crudDeleteAction successfully, model = '.$m->_table. ', deleted id = '.$this->_params['id'],MYMVC_LOG);
			}
		}
		$this->_view->_sendJsonResponse($result);
		return false ; 
	}
	
}
