<?php
/*
Copyright (C) 2009- 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * 
 * Class    :	 	MyMVC_Datasource
 * Copyright:		Auguste 2009
 *
 * 16/07/09 - add logging debug
 * 09/10/09 - add loggin if no datasource processed
 * 11/11/09 -	add controler and action param when calling dsfunc
 * 13/01/10 - _set(), _setFuncs()
 * 23/12/10 - remplace controler par controler
 * 03/12/12 - modification syntax datasources.ini
 * 08/04/14 - Optimisation code
 * 
 * 
 */

define ('MyMVC_DATASOURCE_VERSION','1.0');

class MyMVC_Datasource {
	private $_datasources ;
	private $dsFuncs ;
	
	/**
	 * init datasource class
	 *
	 * @param array $pDsConfig
	 */
	public function __construct($pDsConfig) {
		//var_dump($pDsConfig);die();
		if( is_array($pDsConfig) and count($pDsConfig)>0) {
			foreach( $pDsConfig as $ds=>$data) {
				// 0 = controler,  1 = action, 2 = func , 3 = varName
				$this->__load( $ds, $data['context'],$data['func']);
			}
		}
	}
	
	/**
	 * Ajoute un Datasource
    * $pControler = "controler:action"
    * possibilité d'utilisé !controler:action pour exclure une action
	 *
	 * @param string $pControler
	 * @param string $pAction
	 * @param string $pFunc
	 * @param string $pVar
	 *
	 * TODO:  check for duplicate
	 *  
	 */
	private function __load($pVar,$pContext,$pFunc) {
      $c = explode(',',$pContext); // extraction des différentes paires controler:action
      foreach($c as $oneContext) {
         $detailContext = explode(':',$oneContext);
         $controler = $detailContext[0];
         $action = $detailContext[1];
         if( substr($controler,0,1)=='!') { // ? exclusion
            $controler = substr($controler,1);
            $access = 0 ; // acces fermé pour une exclusion
         } else {
            $access = 1 ;
         }
         //l("$pVar-$controler-$action-$pFunc-$access");
         $this->_datasources[] = array(
                  'varName'=>$pVar,
						'controler'=>$controler,
						'action'=>$action,
						'func'=>$pFunc,
                  'access'=>$access);
      }
           
      /*
      $controlers = explode('|',$pControler);
      $actions = explode('|',$pAction);
		if( $pFunc != '' ) 
		{
			foreach($controlers as $c) {
            if( substr($c,0,1) == '!' ) { // exclusion
               $c = substr($c,1,60);
               $controlerOFF = true;
            }
            foreach($actions as $a) {
                if( substr($a,0,1) == '!' ) { // exclusion
                 $a = substr($a,1,60);
                  $actionOFF = true;
               }
               //echo "$c - $a ";
               $this->_datasources[] = array(
                   'varName'=>$pVar,
						'controler'=>$c,
						'action'=>$a,
						'func'=>$pFunc,
                  'controlerOff'=>$controlerOFF,
                  'actionOff'=>$actionOFF);
            }
         }
		} */
	}
	
	/**
	 * exec datasources methode for a controler/action if exist
	 *
	 * @param object Controler  $pControler
	 * @param string $pAction
	 * 
	 * @return bool
	 * 
	 * TODO:  process datasource for all action of a controler if action == ''
	 * 
	 */
	public function _process( $pControler, $pAction) {	
      //d($this->_datasources);
		$mymvc = MyMVC::_getInstance() ;
		if( count($this->_datasources) > 0) {
         $dsToLoad = array();
			foreach( $this->_datasources as $dsData ) {
            $varName = $dsData['varName'];
            $func = $dsData['func'] ;	
				if( ($dsData['controler'] == $pControler->_name or $dsData['controler'] == 'ALL' ) and
                 ($dsData['action'] == $pAction or $dsData['action'] == 'ALL')) { 
                  if( array_key_exists($varName,$dsToLoad) ) {
                     $dsToLoad[$varName]['load'] *= $dsData['access']; // mise à jour du status
                  } else {
                     $dsToLoad[$varName] = array('func'=>$func,'load'=>$dsData['access']);
                  }
            }
         }
         $dtLoaded = 0 ;   
         foreach( $dsToLoad as $dsVar=>$oneDs) {
            $n = $pControler->_name;
            if($oneDs['load']==1) {
               $func = $oneDs['func'];
               if( method_exists( $this->dsFuncs , $func) ) {
                  $mymvc->_log(MYMVC_DEBUG,'Processing datasource '.$func.'() => $'.$dsVar,MYMVC_LOG); 
                  $pControler->$dsVar  = $this->dsFuncs->$func($pControler->_name, $pAction) ;
                  $dtLoaded++;
               } else {
                  $mymvc->_log(MYMVC_ERROR,"Datasource function : $func() not found !",MYMVC_LOG);
               } 
            }
         }
         //var_dump($pControler->listeCompteActif);
         if( $dtLoaded == 0 ) {
            $mymvc->_log(MYMVC_DEBUG , "No datasource for ".$pControler->_name."/$pAction" , MYMVC_LOG);
         } 
      } else {
			 $mymvc->_log(MYMVC_INFO,"No datasource to process ",MYMVC_LOG);
		}
		return true;
	}
	
	/**
    * 
	 * load class dsFunc here with all methods
	 *
	 * @param dsFunc instance $pFuncs
	 * 
	 * TODO check is $pFuncs is a instance of DsFuncs
	 */
	public function _setFuncs($pFuncs) { $this->dsFuncs = $pFuncs ; }
	
}