<?php
/*
Copyright (C) 2007 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * 
 *  
 * Class    :     MyMVC_DbObject
 * Copyright:     Auguste 2007-2014
 * 
 * 06/10/07 - _getXml pour retourne objet au format XML simple (<nomcol>valeur</nomcol>)
 * 12/10/07 - _update($pData)
 * 14/10/07 - _hasChilds()
 * 17/10/07 - modif appel class DbQuery() pour utiliser myMVC->dbConfig
 * 04/09/09 - _save() => save(), _delete() => delete()
 * 07/11/09 - check if _isDeletable() before delete record in db
 * 13/11/09 - $lastQuery =>  $_lastQuery
 * 07/01/10 - set default values, insert mode adn if value = '' or not set (ex unchecked checkbox)
 * 13/01/10 - _listCols, _errors
 * 15/01/10 - _newid() (dbQuery),
 * 30/01/10 - fix bug in __set() to check if col exists in tableStruct of model, _saveNew()
 * 						suppr _listCols
 * 03/02/10 - _updateSpecialCols()
 * 20/02/10 - _validDbObject() call isValid() with $this->_data param
 * 23/04/10 - _getErrors()
 * 27/04/10 - fix bug  appel  $db->_getLastQuery()
 * 29/04/10 - traite la sérilalisation des données
 * 15/06/10 - ajout log info colonne non valide dans MYMVC_LOG
 * 23/03/12 - _updateFromArray() permet de faire un update sur une liste de cols limités sans aucun formatage
 * 			  meilleur perf aucun traitement n'est effectué.
 * 25/10/12 - fix bug _conserveCols non déclarée
 * 22/02/13 - _toJSON()
 * 11/03/13 - _save(), maj pour tenir compte lors d'un insert des PK non autoIncrementées
 * 11/04/13 - fix bug _populate() qui ne filtrait pas les colonnes non présentes dans la table
 * 12/04/13 - Modif __set() pour accepter de setter des colonnes non présentent dans la table, seront supprimée lors du save()
 * 21/04/13 - Ajout appel afterSave() du model si methode existe après un save()
 * 25/07/13 - Ajout appel beforeDelete() dans _delete();
 * 18/02/14 - Ajout __log(level,message)
 * 12/03/14 - Appel afterSave() dans _save() si exec OK
 * 13/03/04 - remplacement $this->__log par MyMVC::_getInstance()->
 * 20/03/14 - recupération du retour de afterSave() 
 * 07/04/14 - ajout appel beforeSave() dans _save(), remplacement _log() par _logSQL() pour les requetes SQL
 * 18/04/14 - utilisation de MyMVC_ERROR pour stocker les messages d'erreurs
 * V 1.0.2
 * 16/05/14 - Ajout $pkVal sur appel beforeWrite() dans _save() pour savoir si c'est un update ou insert
 * 06/10/14 - Ajout erreur si beforeDelete() retourne false 
 * 20/10/14 - ajout appel afterDelete() dans _delete()
 * 
 * 
 * TODO
 * 
 * Copie des infos du model utile et suppresion du lien vers l'objet model parent
 * gestion des relations dans _save et _delete
 * ajout methode  _link() et unlink(), _findChilds(), _findParent() 
 * proteger la modif de l'ID 
 * _saveNew()  to create a new record from another -> reset pkey, fromdb = false
 * 
 * verifier fonctionnement de _populate()
 * 
 * // extends DbFunc ?? ou u1 seule classe ??	
 * 
*/
define ('MyMVC_DBOBJECT_VERSION','1.02') ; // 16/05/14

class MyMVC_DbObject { 

	public		$_model ;					// objet model pour la table de la BDD
	//public		$_errors ;				// tableaux des erreurs
	public		$_isModified ;		// true si l'objet à été modifié depuis sa lecture en BDD
	public		$_lastQuery ;			// dernière requête SQL exécutée
	
	private		$_dbConfig ;			// config de connexion à la BDD
  private		$_conserveCols ; 
	private		$_data ;					// tableau  col=>val  correspond au champ de la BDD
	public		$_fromDb ;				// true si l'objet vient de la BDD	
	
	public function __construct( $pModel = null , $pData = null  ) {
		if( $pModel === null ) 
		{
			throw new Exception(MyMVC::getTrad('mymvc_DbObjectConstructParametreIncorrect'));
		}
		$this->_conserveCols	= array();
		$this->_model					= $pModel ;
		$this->_fromDb				= false ;
		//$this->_errors				= array();
		if( is_array($pData) ) {
			$this->_populate($pData) ;
			//echo $this->_fromDb;
		} 
		$this->_dbConfig = MyMVC_Config::_getInstance()->dbConfig ;
	}

	public function _setFromDb($pBool=null) {
		if( $pBool === null ) {
			throw new Exception("MyMVC_DbObject::_setFromDb() valeur incorrecte !");
		}
		$this->_fromDb = $pBool;
	}
    
	/*
	 * 
	 */
	public function __set($pCol,$pVal) {
		$this->_data["$pCol"] = $pVal ;
		$this->_isModified = true ;
	}

	/**
	 * Used to add special cols to current dbObject
	 * this cols are not stored in DB only used to display data
	 * 
	 */
	public function OFF_updateSpecialCols() {
		foreach($this->_model->_specialCols as $col=>$func) {
			$this->_data[$col] = $this->_model->$func($this);
		}
	}
    
	/*
	 * idem __set pour recup valeur proprieté
	 * 
	 */
	public function __get($pCol) {
		$val = null;
		if( $pCol != '' and $this->_data != null and array_key_exists( $pCol, $this->_data ) ) {
			$val = $this->_data["$pCol"] ;
		} else {
			if( array_key_exists($pCol, $this->_model->_specialCols)) {
				$val = $this->_data["$pCol"] ;
			}
		}
		return $val;
	}

	/**
	 * retourne la liste des erreurs 
	 * avec $separator comme séparateur
	 *
	 * @param string $separator
	 * 
	 * @return string
	 */
	public function _getErrors($separator = '<br/>') {
		return implode($separator,MyMVC_ERROR::_getInstance()->_getErrors());
		//return implode($separator,$this->_errors);
	}
   
	public function _setError($pError) {
		MyMVC_ERROR::_getInstance()->_addErro($pError);
	}
	
	/*
	 * recharge l'objet en cours depuis la base
	 * si l'objet vient de la base !  fromDb = true ;
	 * 
	*/ 
	private function _reload() {
		$pk = $this->_model->_key ;
		if( $this->_fromDb === false) return true ;  // pas de rechargement, mais pas d'erreur
		$model = $this->_model->_name ;
		$m = new $model();
		$o = $m->_find( $this->_data[$this->_model->_key]) ;
		if( $o === false ) {
			throw new Exception("MyMVC_DbObject::_reload() -> Model::_find()") ;
		}
		$this->_populate($o->_getArray()) ;
		$o = null ;
		return true ;
	}
    
	/**
	 * save current object to a new one
	 * set fromDb = false, and reset pkey
	 * 
	 * @return unknown_type
	 */
	public function _saveNew() {
		$this->_fromDb = false;
		$this->_isModified = true ;
		return $this->_save();
	}
    
	/**
	 * save current object with value from array
	 * ignore _data
	 * useful for updating one cols in dd, 
	 * can only be used for update (no insert)
	 * use current pk
	 *  
	 * @param array $data
	 * 
	 */
	public function _updateFromArray($data) {
		 // TODO check $data param
		foreach($data as $col=>$val) {
			if( array_key_exists($col, $this->_model->_tableStruct)) { // and $col != $this->_model->_key )
					$dataSql["$col"] = $val ;
			}
			$pk = $this->_data[$this->_model->_key];
			if( $pk === null ) {
				throw new Exception("MyMVC_DbObject::save() valeur PrimaryKey non trouvée!");
		}
		$db = new MyMVC_DbQuery( $this->_dbConfig );
		$ret = $db->_update($this->_model->_table , $this->_model->_key , $pk , $dataSql ) ;
		if( $ret === true ) {
			$this->_isModified = false ;
		}
		// TODO trap error of db->_update()
		}
	}
     
	/**
	 * 
	 * Fonction save pour l'objet en cours
	 * Si fromDb == false execute un insert, sinon un update
	 * 
	 * Test si l'objet est valide avec isValid()
	 * Execute les fonctions de formattage  beforeWriteFuncs
	 * Affecte les valeurs par défaut (uniquement pour les insert)
	 * Sérialise les colonnes (définit comme sérialisables)
	 * 
	 * 
	 */
	public function _save() {
		#### ajouter controle s'il y a des errors
		#### dans ce cas pas de save => return false

		$this->_errors = array() ; // supprime erreurs précédentes
		$pkVal		= null;
		$dataSql	= array() ;
		$pkName		= $this->_model->_key ;
		$ret			= false ;
      
		if( $this->_model->beforeSave($this) ) {
			if( $this->_validDbObject() ) {
				// construction de la liste des colonnes de la BDD
				foreach($this->_model->_tableStruct as $c=>$colInfo) {
					if( $c !=  $pkName ) { // on ajoute pas la PK 
						if( isset($this->_data[$c]) ) { $dataSql["$c"] = $this->_data[$c] ; } // ne sauve que les colonnes existant dans l'objet
					} else {
						$pkVal = $this->$c ;  // recup de la valeur de la PK
					}
				}    		
				// formatage des valeurs avant enregistrement
				if( $this->_model->_applyReadWriteFunc) { 
					$dataSql = $this->_model->beforeWrite($dataSql,$pkVal);
				}
				// default values
				foreach( $this->_model->_defaultValueFuncs as $col=>$a ) {
					// default value apply to empty value before insert
					// defautValues are not set in update mode
					if( (!array_key_exists($col, $dataSql) or $dataSql[$col] == '' ) AND ( $this->_fromDb == false ) ) {
						// use func to set default value
						$f = $a[0] ;
						if( $a[1] ) {
							$dataSql[$col] = $this->_model->$f($col);  // model method
							//loginfo("Default val for $col " . $dataSql[$col]);
						} else {
							$dataSql[$col] = $f($col);  // standalone func
						}
					}
				}
				## serialiszation de valeurs
				foreach( $this->_model->_serializedCols as $col) {
					if( isset($dataSql[$col]) and $dataSql[$col]!='') { $dataSql[$col] = serialize( $dataSql[$col] ); }
				}
				// on fait rien si erreur ou pas de data a enregistrer
				if( count($dataSql)==0 ) { // pas de save , non bloquant
					MYMVC::_getInstance()->_log(MYMVC_DEBUG,MyMVC::getTrad('mymvc_saveAnnuleNoData'),MYMVC_LOG);
					$ret = false ;
				} else {
					$db = new MyMVC_DbQuery( $this->_dbConfig );
					if( $this->_fromDb === true ) {
						 // update  			
						 if( $pkVal === null ) { throw new Exception("MyMVC_DbObject::save() valeur PrimaryKey non trouvée!"); }
						 // TODO ajouter test isModified = true
						 $ret = $db->_update($this->_model->_table , $pkName , $pkVal , $dataSql ) ;
						 if( $ret === true ) { $this->_isModified = false ; }
						 // TODO trap error of db->_update()
					} else {
						 // insert
						 // si pk non autoincremente il faut fournir la PK en requete insert
						 if( !$this->_model->_keyAutoIncremente ) {
							 $this->$pkName = $pkVal;
							 if( $pkVal === null ) { throw new Exception("MyMVC_DbObject::save() valeur PrimaryKey non trouvée pour insert ! "); }
							 $dataSql[$pkName] = $pkVal;
						 } 
						 $ret = $db->_insert($this->_model->_table ,$dataSql ) ;
						 if( $ret === true ) {
							 //recup de l'id créé
							 if( $this->_model->_keyAutoIncremente ) { $this->$pkName = $db->_getNewId(); } // recup id si autoIncrement uniquement
							 $this->_fromDb = true ; // l'enregistrement est en base maintenant
							 $this->_isModified = false ;
						 }
					}
					$this->_lastQuery = $db->_getLastQuery() ;
					MyMVC::_getInstance()->_log(MYMVC_SQL,$this->_lastQuery,MYMVC_LOG);
				}
			} else { // validDbObject()
				MyMVC::_getInstance()->_log(MYMVC_DEBUG,MyMVC::getTrad('mymvc_dbObjectNonValideSaveAnnule'),MYMVC_LOG);
			}
		} else { // beforeSave()
			MyMVC::_getInstance()->_log(MYMVC_DEBUG,MyMVC::getTrad('mymvc_saveAnnuleParBeforeSave'),MYMVC_LOG);
		}
		if($ret) {   
			$ret = $this->_model->afterSave($this); // appel methode afterSave() uniquement si pas d'erreur
		}
		return $ret ;
	} 
    
	public function _delete() {
		$this->_errors = array(); // suppression anciennes erreurs
		$ret = false ;
		if( $this->_fromDb ) { // undeletable if not from db
			$pkName = $this->_model->_key ;
			if( $this->$pkName !== null ) {
				if( $this->_model->isDeletable($this->$pkName) ) { // if not deletable, error is stored in model directly
					// call beforeDelete()
					if( $this->_model->beforeDelete($this->$pkName) ) {
						$object = $this->_model->_find($this->$pkName); // backup des valeurs pour appeler afterDelete()
						$db = new MyMVC_DbQuery($this->_dbConfig) ;
						$ret = $db->_delete($this->_model->_table , $pkName , $this->$pkName ) ;
						$this->_lastQuery = $db->_getLastQuery() ;
						MyMVC::_getInstance()->_log(MYMVC_SQL,$this->_lastQuery,MYMVC_LOG);
						$this->$pkName = null ;  // l'objet devient new, 
						$this->_fromDb = false ; // sans relation avec la base
						if($ret) {
							$this->_model->afterDelete($object->_toArray());
						}
					} else {
						if( MYMVC_ERROR::_getInstance()->_getLastError() == '') {
							MyMVC_ERROR::_getInstance()->_addError('Suppression '.$this->_model->_table.' impossible (bloquée par beforeDelete())');
						}
					}
				} else {
					if( MYMVC_ERROR::_getInstance()->_getLastError() == '') {
						MyMVC_ERROR::_getInstance()->_addError('Suppression '.$this->_model->_table.' impossible (bloquée par isValid())');
					}
				}
			} else {
				if( MYMVC_ERROR::_getInstance()->_getLastError() == '') {
					MyMVC_ERROR::_getInstance()->_addError('Suppression '.$this->_model->_table.' impossible: enregistrement non trouvé dans la base de données !');
				}
			} 
		} else {
			if( MYMVC_ERROR::_getInstance()->_getLastError() == '') {
				MyMVC_ERROR::_getInstance()->_addError('Suppression '.$this->_model->_table.' impossible : cet enregistrement ne vient de la base de données !');
			}
		}
		return $ret ;	
	}
    
	/**
	 * return data of DbOject in array format
	 * array('col'=>'value',...)
	 * 
	 * @return array
	 * 
	 */
	public function _toArray() { return $this->_data ; }
    
	/**
	 * 06/10/07
	 * Retourne l'objet au format JSON
	 * {"colName":"colVal",....}
	 * 
	 * @return string 
	 */

	public function _toJSON() {
		$this->_updateSpecialCols(); 
		$json = '{' ;
		foreach( $this->_data as $col=>$val) {
			$json .=  '"'.$col.'":"'.$val.'",' ;
		}
		$json = substr($json,0,-1); // suppression dernière virgule 
		$json .= '}';
		return $json ;
	}
    
	/**
	 * 06/10/07
	 * 
	 * @return string
	 */
	public function _toXML() {
		$this->_updateSpecialCols(); 
		$xml = '' ;
		foreach($this->_data as $col=>$val) {
			$xml .=  "<$col>$val</$col>" ;
		}
		return $xml ;
	}
    
	/* 
	 *  
	 * 26/03/13
	 * 
	 * Construit un objet à partir d'un tableau de nom/valeur
	 * les fonctions de formatage afterRead... ne sont pas appliquées
	 * 
	*/
	function _populate($pData=null) {
		if( $pData === null or !is_array($pData) ) {
			throw new Exception(MyMVC::getTrad('mymvc_DbObjectPopulateParametreIncorrect'));
		}
		$this->_fromDb = false ;
		foreach( $pData as $c=>$v) {
			if( $c == $this->_model->_key and $v != '') {
				$this->_fromDb = true ; // primary key found -> means update
			} 
			$this->$c = $v ; // 'remplissage du dbObject
		}
		// si pas de PK => valeur defaut = null;
		if( !$this->_fromDb ) $this->_data[$this->_model->_key] = null ;
	}
    
	/** 
	 * Appel les fonctions de validations definies dans le model
	 * pour chaque colonnes de l'objet 
	 * retourne True si toutes les colonnes sont OK
	 * sinon False
	 * 
	 * Si $pCol == null teste toutes les cols de l'objet
	 * sinon test uniquement la colonne $pCol
	 * 
	 * @param string $pCol
	 * @return bool
	 * 
	 */
	public function _validDbObject($pCol = null) {
		$valid = true ;
		if( $pCol === null ) { // test toutes les cols
			foreach( $this->_model->_tableStruct as $col=>$colInfo ) {
				if( array_key_exists($col, $this->_data) ) { // ne check que les colonnes dans l'objet
					$v = $this->_model->isValid($col,$this->_data) ;
					if( $v !== '' ) {
						$valid = false ;
						//$this->_errors[] = $v ; // message d'erreur
						MyMVC_ERROR::_getInstance()->_addError($v);
						//MyMVC::_getInstance()->_log(MYMVC_ERROR,"Colonne $col non valide", MYMVC_LOG);
					}
				}
			}
		} else {
			// test une seule colonne
			$v = $this->_model->isValid($pCol, $this->_data[$pCol]) ;
			if( $v !== '' ) {
				$valid = false ;
				$this->_errors[] = $v ;
			}
		}
		//if( $valid ) MyMVC::_getInstance()->_log(MYMVC_INFO,'--- dbObject Valid');
		//if( !$valid ) MyMVC::_getInstance()->_log(MYMVC_INFO,'--- dbObject not valid');
		return $valid ;
	}
}
