<?php
/*
Copyright (C) 2007 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * 
 * 
 * Class  DbQuery() 
 *  
 * 12/10/07 - fix buf _delete()
 * 13/11/09 - $lastQuery =>  $_lastQuery
 * 13/01/10 - add define('SQL_XXX..
 * 15/01/10 - _newId, _nbLines, replace die() with exceptions
 * 20/01/10 - fix error in params order new DbfuncLib()
 * 						_getNewId(), _getLastQuery(), _getNbLines() 
 * 18/03/10 - _updateBySql()  maj format param $values ;
 * 21/05/10 - code pour sqllite
 * 09/07/10 - _sum()
 * 21/03/11 - check get_magic_quotes_gpc() avant d'utiliser addslashes();
 * 25/02/14 - optimisation code
 * 08/04/14 - Mise à jour Traduction 
 * 
 *   
 */
define ('MyMVC_DBQUERY_VERSION','1.0');

// SQL templates
define('SQL_INSERT','INSERT INTO {TABLE} ({COLS}) VALUES({VALUES})') ; 			  
define('SQL_UPDATE','UPDATE {TABLE} SET {VALUES} WHERE {WHERE} {OPTION}') ; 	
define('SQL_DELETE','DELETE FROM {TABLE} WHERE {WHERE} {OPTION}');
define('SQL_STRUCT_TABLE','SHOW COLUMNS FROM {TABLE}') ;	
define('SQL_FIND','SELECT {SELECT} FROM {TABLE} WHERE {WHERE} {OPTION}');
define('SQL_TRUNCATE','TRUNCATE TABLE {TABLE}'); // !! ne fonctionne pas sur sqllite !!
define('SQL_TRUNCATE_SQLITE','DELETE FROM {TABLE} WHERE 1'); 
define('SQL_COUNT','SELECT COUNT(*) AS nbLines FROM {TABLE} WHERE {WHERE} ');
define('SQL_SUM','SELECT SUM({COL}) AS total FROM {TABLE} WHERE {WHERE} ');

class MyMVC_DbQuery {
		
	public $_db = null ;
	private $_driver ; 
		
    public function __construct( $pDbConfig = null) {
    	if( $pDbConfig === null or !is_array($pDbConfig) ) {
    		throw new Exception(MyMVC::getTrad('mymvc_DbQueryContructParametreDbConfigIncorrect'));
    	}
    	// les fichiers db sqllite doivent être dans le dossier de l'application
    	if( $pDbConfig['driver'] == 'sqllite' ) {
    		$pDbConfig['db'] = BASE_PATH.$pDbConfig['db'] ;
    	}
    	$this->_db = new DbfuncLib( 
    		$pDbConfig['host'], 
    		$pDbConfig['user'], 
    		$pDbConfig['password'], 
    		$pDbConfig['db'], 
    		$pDbConfig['driver'] ) ;
    	$this->_driver = $pDbConfig['driver']; // on garde le driver
    }
		
	/*
	 * supprime une ligne dans la table
	 * du model
	 */
	public function _delete($pTable=null,$pPrimaryKey=null,$pVal=null) {
		if( $pTable === null or $pPrimaryKey === null or $pVal===null 
		or $pTable == '' or $pPrimaryKey == '' or $pVal == '') {
			throw new Exception(MyMVC::getTrad('mymvc_DbQueryDeleteParametreIncorrect'));
		}
		$sql 	= str_replace('{TABLE}',$pTable,SQL_DELETE) ;
		$sql 	= str_replace('{WHERE}', "$pPrimaryKey='$pVal'", $sql);
		$sql 	= str_replace( '{OPTION}' , ' limit 1' , $sql ) ;
		$ret = $this->_db->execSql($sql,'',2); // requete delete
		return $ret ; 
	}

	/**
	 * count record in one table
	 *
	 * @param unknown_type $pTable
	 * @param unknown_type $pWhere
	 * @return unknown
	 */
	public function _count($pTable = null , $pWhere = '' ) {
		if( $pTable == '' or $pTable === null ) {
    		throw new Exception(MyMVC::getTrad('mymvc_DbQueryCountParametreIncorrect'));
    	}
		$ret = 0 ;
		$sql = str_replace('{TABLE}',$pTable,SQL_COUNT);
		if( $pWhere == '' ) {
			$sql = str_replace('{WHERE}','1',$sql);
		} else {
			$sql = str_replace('{WHERE}',"$pWhere",$sql);
		}
		if( $this->_db->execSql($sql,'',0,'nbLines') ) { // select avec recup de la colonne total directement
			$ret = $this->_db->_result ;
		} else {
			throw new Exception("MyMVC_DbQuery::_count()  erreur execution sql");
		}
		return $ret ;
	}

	/** sum une colonne avec un where optionnel */
	public function _sum($pTable, $pCol, $pWhere=null) {
		if( $pTable == null or $pCol == null or $pCol == '' or $pTable =='') {
			throw new Exception(MyMVC::getTrad('mymvc_DbQuerySumParametreIncorrect'));
		}
		$sql = str_replace('{TABLE}',$pTable, SQL_SUM);
		$sql = str_replace('{COL}',$pCol,$sql);
		if( $pWhere != null) {
			$sql = str_replace('{WHERE}',$pWhere, $sql);
		} else {
			$sql = str_replace('{WHERE}','1', $sql);
		}
		if( $this->_db->execSql($sql,'',0,'total') ) { // select avec recup de la colonne total directement
			$ret = $this->_db->_result ;
		} else {
			throw new Exception(MyMVC::getTrad('mymvc_DbQuerySumErreurSQL'));
		}
		return $ret ;
	}
	
		/*
     * update un enregistrement dans la base
     * $pData contient des nom/valeurs pour les colonnes de la table
     * les valeurs ont été validées et formatées au préalable
     * 
     */
    public function _update($pTable=null, $pPrimaryKey=null, $pValPk=null, $pData=null) {
    	if( $pTable == '' or $pTable === null or !is_array($pData) or $pPrimaryKey === null 
    	or $pPrimaryKey == '' or $pValPk === null or $pValPk == '') {
    		throw new Exception(MyMVC::getTrad('mymvc_DbQueryUpdateParametreIncorrect'));
    	}
    	
    	$values= '' ;
    	foreach($pData as $n=>$v) {
    		// protection des donnnées si magic_quote OFF
    		if(!get_magic_quotes_gpc()) $v = addslashes($v);
    		// les val num passé entre '' seront convertie en num par mysql
    		$values .= ($values=='')? "$n='$v'" : ", $n='$v'" ; // construit la liste values('val1','val2'..)
    	}  	
    	$sql = str_replace( '{TABLE}' , $pTable , SQL_UPDATE ) ;
    	$sql = str_replace( '{VALUES}' , $values , $sql ) ;
    	$sql = str_replace( '{OPTION}' , ' limit 1' , $sql ) ;
    	$sql = str_replace ( '{WHERE}' , "$pPrimaryKey='$pValPk'", $sql);
    	$ret = $this->_db->execSql($sql,'',2) ; // requete update
    	return $ret ;
    }
    
    
    public function _insert($pTable=null, $pData=null) {
    	if( $pTable == '' or $pTable === null or !is_array($pData)) 
    	{
    		throw new Exception(MyMVC::getTrad('mymvc_DbQueryInsertParametreIncorrect'));
    	}
    	$cols		=	''	;
    	$values	=	''	;
    	foreach( $pData as $c=>$v ) {
    		// protection des donnnées si magic_quote OFF            
         if( !get_magic_quotes_gpc() ) $v = addslashes($v);
    		$cols .= ($cols=='')? "$c" : ", $c" ; // construit la liste set(col1,col2..)
    		// les val num passé entre '' seront convertie en num par mysql
    		$values .= ($values=='')? "'$v'" : ", '$v'" ; // construit la liste values('val1','val2'..)
    	}
    	$sql = str_replace( '{TABLE}' , $pTable , SQL_INSERT) ;
    	$sql = str_replace( '{COLS}' , $cols , $sql ) ;
    	$sql = str_replace( '{VALUES}' , $values , $sql ) ;
    	$ret = $this->_db->execSql($sql,'',1) ; // requete insert
    	return $ret ;
    }
    	
    ################################################################################################
    /*
     * 
     * execute une requete SQL 
     * retour true si trouvé > 0, sinon false
     * erreur SQL = arrêt
     * 
     * recupere la liste des colonnes renvoyée par la requete et les places
     * dans _listCols
     * si au moins une colonne n'existe pas dans la structure
     * l'enregistrement est désactivé
     * 
     */  
    public function _query( $pSql=null ) {
    	if( $pSql === null )  throw new Exception(MyMVC::getTrad('mymvc_DbQueryQueryParametreIncorrect'));
    	$ret = false ;
    	if( $this->_db->execSql($pSql) ) {  // requete qui renvoie un resultset, select, show...
    		if( $this->_db->_nbLines > 0 ) {
    			$ret = $this->_db->_result ;
    		} else {
    			$ret = array();
    		}
    	} else {
    		throw new Exception(MyMVC::getTrad('mymvc_DbQueryQueryErreurSQL').$this->_db->_err ) ;
    	}
    	return $ret ;
    }
    
    public function _getLastQuery() { return $this->_db->_lastQuery ; }
    
    public function _getNbLines() { return $this->_db->_nbLines ; }
    
    public function _getNewId() { return $this->_db->new_id; }
    
    /**
     * 
     * Exécute un requete update sur la base
     *
     * @param string $pTable
     * @param string $pWhere
     * @param array $pSet
     * 
     * @return bool
     * 
     */
    public function _updateBySql($pTable=null, $pWhere=null, $pSet=null) {
    	if($pTable===null or $pWhere===null) {
    		throw new Exception(MyMVC::getTrad('mymvc_DbQueryUpdateBySQLParametreIncorrect'));
    	}
    	$sql = str_replace('{TABLE}',$pTable, SQL_UPDATE) ;
    	$sql = str_replace('{WHERE}',$pWhere, $sql) ;
    	$values = '';
    	foreach($pSet as $key=>$val) {
    		// protection des donnnées si magic_quote OFF
    		if(!get_magic_quotes_gpc()) $val = addslashes($val);
    		$values .= "$key='$val'," ;
    	}
    	$values = substr($values,0,-1); // supprime dernière virgule en trop !
    	$sql = str_replace('{VALUES}', $values, $sql) ;
    	$sql = str_replace('{OPTION}','', $sql) ;
    	$ret = $this->_db->execSql($sql,'',2); // req update
    	return $ret;
    }
    
    public function _deleteBySQl($pTable=null, $pWhere=null) {
    	if($pTable===null or $pWhere===null) throw new Exception(MyMVC::getTrad('mymvc_DbQueryDeleteBySQLParametreIncorrect'));
     	$sql = str_replace('{TABLE}',$pTable, SQL_DELETE) ;
    	$sql = str_replace('{WHERE}',$pWhere, $sql) ;
    	$sql = str_replace('{OPTION}','', $sql) ;
    	$ret = $this->_db->execSql($sql,'',2); // req delete
    	return $ret;
    }
    
    public function _truncateTable($pTable=null) {
    	if($pTable===null) throw new Exception(MyMVC::getTrad('mymvc_DbQueryTruncateTableParametreIncorrect'));
    	if( $this->_driver == 'sqlite' ) {
    		$sql = str_replace('{TABLE}', $pTable, SQL_TRUNCATE_SQLITE) ;
    	} else {
    		$sql = str_replace('{TABLE}', $pTable, SQL_TRUNCATE) ;
    	}
    	$ret = $this->_db->execSql($sql,'',2); // req delete
    	return $ret;
    }
    
    public function _getTableStruct($pTable) {
    	if($pTable===null) throw new Exception(MyMVC::getTrad('mymvc_DbQueryGetTableStructParametreIncorrect'));
      $sql = str_replace('{TABLE}', $pTable, SQL_STRUCT_TABLE);
      $ret = false ;
      $res = $this->_query($sql) ;
      if( count($res) > 0 ) {
         $ret = $res ;
      }
    	return $ret ;
    }
    
}
