<?php
/*
Copyright (C) 2007 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

#################################################################
 * 
 * Display a error page
 * also log error to  BASE_PATH/logs/error.log
 * 
 * 06/10/07 - maj pour utilisation de MyMVCPath
 * 20/05/09 - major update MyMVC 1.0
 * 16/07/09 - add logging error to mymvc.log
 * 17/07/09 - check if templateLib is loaded, add displayException()
 * 30/10/09 - renommage methode =>  prefix _
 * 15/01/10 - suppresion utilisation template, code html inclut dans la classe, ainsi l'affichage des erreurs est autonome
 * 20/01/10 - _displayException()
 * 10/02/11 - corrections mineures
 * 15/03/13 - Ajout test AJAX pour retour erreur Ajax
 * 08/04/14 - Optimisation code, mise à jour traductions
 * 18/04/14 - Ajout gestion des erreurs globales, utilisé par toutes les classes
 * V1.11
 * 24/06/14 - ajout log des erreurs dans _addError()
 * 
*/

define ('MyMVC_ERROR_VERSION','1.11'); // 18/04/14

class MyMVC_Error {
   
	private $_errors		= array();
	static	$_instance	= null;
	
	static public function _getInstance() {
 		if(self::$_instance == NULL ) {
          self::$_instance = new MyMVC_Error();
     	}
     	return self::$_instance;
 	}
	
  public function __construct() { }
  
	// gestion des erreurs en global
	public function _addError($pError) {
		if( $pError != '' ) {
			$this->_errors[] = $pError ; 
			$this->_log($pError);
		}
	}
	public function _resetError() { $this->_errors = array() ; }
	public function _getErrors() { return $this->_errors ; }
	public function _getLastError() {
		$n = count($this->_errors)-1 ;
		return $this->_errors[$n] ;
	}
	
	/** affichage d'une erreur/exception */
	public function _show( $pLine , $pError , $pFile = '' , $pSource = '', $pTrace = '' ) {
		global $errorPage;	
		//$this->_log( "$pError in $pSource from $pFile") ;
		$errorPage = str_replace('{ERROR_REPORT}',$pError,$errorPage);
		$errorPage = str_replace('{ERROR_SRC}',$pSource,$errorPage);
		$errorPage = str_replace('{FILE}',$pFile,$errorPage);
		$errorPage = str_replace('{ERR_LINE}',$pLine,$errorPage);
		$traceData = '' ;
		if(is_array($pTrace) ) {
			foreach($pTrace as $one) {
				$traceData .= '<tr class="tr_data"><td>'.$one['file'] .'</td><td>'.$one['line'].'</td>'.
						'<td>'.$one['class'].'->'.$one['function'].'()</td>'.
						//'<td>'.$one['type'].'</td>'.
						'<td>-</td></tr>'."\n" ;
			}
		}
		$errorPage = str_replace('{TRACE_DATA}',$traceData,$errorPage);
		if( true===MyMVC_AJAX) { // appel Ajax
			$r = array('requestOk'=>false,'message'=>$pError);
			print json_encode($r) ;
			exit(1);
		}
		print $errorPage;
		exit(1) ;
	}
    
	/**
	 * Affichage d'une exception avec les infos du trace
	 * Retour JSON si l'erreur se produit lors d'un appel AJAX
	 * 
	 * @param Exception $pException
	 */
	public function _displayException(Exception $pException) {
		if( true===MyMVC_AJAX ) { // définit grace au param  MyMVC_AJAX = 1 dans les appels Ajax
			$result=array('requestOk'=>false,'message'=>MyMVC::getTrad('mymvc_erreurTraitement').'<br/><br/>'.$pException->getMessage());
			header('Cache-Control: no-cache, must-revalidate');
			header('Content-type: text/html');
			print json_encode($result) ;          
			exit; 
		} else {
			// TODO LOG Error
			$this->_log($pException->getMessage());
			$this->_show( $pException->getLine(), $pException->getMessage(), $pException->getFile(), MyMVC::getTrad('mymvc_exception'), $pException->getTrace() ) ;
		}
	}
    
	/**
	 * 01/04/07
	 * log les messages d'erreur
	 */
 	private function _log($pError) {
		MyMVC::_getInstance()->_log( MYMVC_ERROR , $pError , ERROR_LOG ) ;	
		return true;
	}
    
}

/** code HTML de la page d'erreur 
 * 
 * TODO : Traduction des textes dans la page d'erreur
 *  */

$errorPage  = <<<FIN

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">

<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<title>Error Report</title>

<style>
H1 {
    background: red; 
    color : white ; 
    border: 1px black solid ;
    text-align: center ;
    width: 100% ;
}

#error { 
    font-size: 12pt ; 
    margin-top: 0px ; 
}

body { background: #FFFFB6 ; color: #000000 ; }


#info { color: blue ; font-style: italic ; font-size: 10pt ; }


TD {
	font-size: 0.8em ;
	}
	
.tr_head { background: red ; text-align: center; }

.tr_data { background: #DCDCDC ; }

.tr_data2 { background: #ABABAB ; text-align: center; }

.tb_trace {
	border: 1px solid #666666 ; 
	padding: 5px;
	}
	
</style>
	
</head>


<body>

<h1>Error Report</h1>

<div id="error">
<p align="center"> 
Error source : <span id="info"> [ {ERROR_SRC} ]</span> </p>

<hr />
Error triggered in file : <span id="info">{FILE}</span><br/> 

</p>
<hr />
 
<p>
Détails : <span id="info">{ERROR_REPORT}</span>
</p>

<hr/>

<table align="center" class="tb_trace" width="95%">

<tr class="tr_head">
<td colspan="5">Exception trace info</td>
</tr>

<tr class="tr_data2">
<td>File</td>
<td>Line</td>
<td>Function</td>
<td>Args</td>
</tr>

<tr class="tr_data">
<td>{FILE}</td>
<td>{ERR_LINE}</td>
<td></td>
<td></td>
</tr>


{TRACE_DATA}

</table>

<hr/>
<p align="center">
<a href="javascript:window.history.go(-1)">Retour</a> - 
<a href="javascript:document.location.reload()">Ré-essayer</a> - 
<a href="home.html">Accueil</a>
</p>
</div>

</body>

FIN;



