<?php
/*
Copyright (C) 2011 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
/**
 * 14/01/14 - création sous forme de lib externe à MyMVC 
 * 
 * 
 * @TODO  setAcls() setGroups() setContexts()
 * 
 */

class MyMVC_Acl {
   
   private $acls;
   
   public function __construct($pType) // type = ini | db
   {
      switch($pType) {
         case 'ini' : $this->__loadAclIni(); break;
         case 'db' : $this->__loadAclDb(); break;
      }
   }
   
   public function setAcls($pAcls) {
      
   }
   
   public function setGroups($pGroups) {
      
   }
   
   public function setContext($pContexts) {
      
   }
   /**
    * Controle si un acces PUBLIC est autorisé
    * 
    * 14/01/14
    * 
    * @param string $pControler
    * @param string $pAction
    * @return boolean
    */
   public function allowPublic($pControler, $pAction) {
     $allowPublic = false;
     foreach($this->acls['allowRules'] as $oneAcl) {
          if( $oneAcl['group']==='PUBLIC' 
              AND ($oneAcl['controler']===$pControler OR $oneAcl['controler'] === 'ALL' )
              AND ($oneAcl['action']==='ALL' OR $oneAcl['action']===$pAction )
             ) {
             $allowPublic = true;
             break;
          }
      } 
      //d($allowPublic);
      return $allowPublic;
   }
   
   /**
    * Controle si user connecté, sinon => renvoi sur page de login
    * si oui vérifie les droits d'accès par rapport à la ressource demandée (controler/action)
    * 
    * @param string $pControler
    * @param string $pAction
    * @param string $pUser
    * 
    * @return bool
    * 
    * 09/01/14  refonte complete de la gestion et traitement des ACLs
    * 
    * 
    */
    public function allowUser( $pControler, $pAction, $pUser )
    {
      $userGroups = array('LOGGED'); // par défaut tous les users sont membres de LOGGED			
      foreach( $this->acls['groups'] as $group=>$members) {
         $membersArray = explode( ',', $members);
         if( in_array($pUser, $membersArray) ) $userGroups[] = $group ; // creation liste des groupe du user
      }
      $userRules = array();
      // lecture allowRules
      //var_dump($this->acls['allowRules'],$userGroups);
      foreach( $this->acls['allowRules'] as $oneAcl) {
         if( in_array($oneAcl['group'],$userGroups) // is user in acl group ? 
            AND ( $oneAcl['controler']===$pControler OR $oneAcl['controler']==='ALL')
            AND ( $oneAcl['action']===$pAction OR $oneAcl['action']==='ALL')
          ) {
            $userRules[$oneAcl['group']]='allow';// on memorise le droit d'accès
         }
      }
      //d($userRules);
      
      foreach( $this->acls['denyRules'] as $oneAcl) {
         //var_dump($oneAcl);
         if( in_array($oneAcl['group'],$userGroups) // is user in acl group ? 
            AND ( $oneAcl['controler']===$pControler OR $oneAcl['controler']==='ALL')
            AND ( $oneAcl['action']===$pAction OR $oneAcl['action']==='ALL')
          ) {
            $userRules[$oneAcl['group']]='deny'; // annule un éventuel allow définit plus haut pour le même groupe
         }
      }
      
      // recherche s'il y a au moins un allow 
      $allowAccess = false;
      foreach($userRules as $grp=>$rule) {
         if($rule === 'allow') { 
            $allowAccess = true;
            break;
         }
      }

      return $allowAccess ;
   }
   
     /**
     * Lecture de config/acl.ini pour créer : 
     *  
     * config['groups']
     * config['rules']
     *
     * @TODO 
     *   Sauver la config dans /cache/acl.mymvc
     * 
     */
    private function __loadAclIni()
    {
      $tmp = array();
      $acls = array();
     
      // aclType = 'file'
      if( is_file( BASE_PATH.'config/acls.ini') )
      {
         $acls = parse_ini_file( BASE_PATH.'config/acls.ini',true);
         // @TODO : Log 
      } else { 
         MyMVC::_getInstance()->_log(MYMVC_ERROR,'config/acls.ini not found',MYMVC_LOG);
         throw new Exception('Error config/acl.ini not found');
      }
    	
    	$this->acls['groups'] = $acls['groups'] ; // liste des groupes avec les membres
		// @TODO Vérifier si count > 0
		$this->acls['allowRules'] = $this->__loadRules($acls['allowRules'],$acls['contexts']);
      $this->acls['denyRules'] = $this->__loadRules($acls['denyRules'],$acls['contexts']);
		unset($tmp);	
    }
    
    
    private function __loadAclDb() {
       // recherche des données ACL dans la base
      $a = new Acl();
        $dbAcls = $a->_findAll('acl_enable="O" ' );
        foreach($dbAcls as $one) {
           switch( $one->acl_type) {
              case 'Groupe': {
                 $this->acls['groups'][$one->acl_name]=$one->acl_value;
                 break;
              }
              case 'Context':{
                 $this->acls['contexts'][$one->acl_name]=$one->acl_value;
                 break;
              }
              case 'Allow':{
                 $this->acls['allowRules'][$one->acl_name]=$one->acl_value;
                 break;
              }
              case 'Deny':{
                 $this->acls['denyRules'][$one->acl_name]=$one->acl_value;
                 break;
              }   
           } // switch
        } // foreach
    }
    
    /**
     * 
     * Extrait toutes les lignes d'ACL depuis $pRules
     * et découpe les lignes de context, pour créer des lignes uniques : 
     *   groupe/controler/action/droit
     * $pPolicy indique le bloc d'acl utilisé (allow ou deny) 
     *
     * @param array $pRules    
     * @param string $pPolicy
     * 
     */
    private function __loadRules($pRules,$pContexts) {
    	$acls = array();
    	foreach( $pRules as $group=>$rule) {
    		$fullRules = array();
    		$aRules = explode(',',$rule); // plusieurs contextes séparée par ,
    		foreach($aRules as $oneRule) {
            //echo $oneRule;
            if( $pContexts[$oneRule] != '' ) { // context exist?
               $c = explode(',',$pContexts[$oneRule]) ; // plusieurs ressources séparées par ,
               $fullRules = array_merge( $fullRules, $c) ;
            } else { } // context ignoré  @TODO log error context non trouvé
    		}
    		if( count($fullRules>0) ) {
    			foreach($fullRules as $oneRule) {
               $ruleData = explode(':',$oneRule); // separation controler action
               $acls[] = array('group'=>$group,'controler'=>$ruleData[0],'action'=>$ruleData[1]); 
            }
    		}	
    	}
		return $acls;
    }
    
}
