<?php
/*
Copyright (C) 2010 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
 * date functions
 * 
 * 16/03/10 - 	_formatToMysql()
 * 23/09/10 - 	getDebutFinForPeriode()
 * 06/10/10 - 	calculBorneDate()
 * 09/03/11 - 	suppression espace en trop dans format 1 et 2 de _formatMysqlToDate()
 * 				fix bug formatage yyyy dans _formatToMysql()
 * 07/03/13 - calculBorneDate()  ajout période  yearMonth
 * 
 * 
 * 
 */
 
class DateLib
{

	/**
	 * Formatte un date au format Mysql en un format prédéfini (ex dd/mm/yyyyy)
	 * 
	 *
	 * @param string $pDateMysql
	 * @param string $pFormat
	 * 
	 * @return string
	 * 
	 * @todo  permettre le choix du format directement avec pFormat = dd/mm/yyyyy par exemple
	 * 
	 */
 	public function _formatMysqlDate($pDateMysql,$pFormat=2)
 	{
 		// convertit une date Mysql YYYY-MM-JJ en date fr suivant format
  	// 1 =  JJ.MM.AAAA  2 =  JJ/MMM/AA   3 =  JJMMAA
  	// 4 = Lundi 4 Novembre 2003   5 = Lundi 4 Novembre
  	// 6 = 4 Novembre
  	$s_Jour = substr($pDateMysql,8,2)+0;
   $s_Mois = substr($pDateMysql,5,2)+0;
  	$s_Annee4 = substr($pDateMysql,0,4);
  	$s_Annee2 = substr($pDateMysql,2,2);
  	$i_JourSemaine = date('w',mktime(0,0,0,$s_Mois,$s_Jour,$s_Annee4))+0;
  	$TI_JourSemaine = array ("Dimanche","Lundi","Mardi","Mercredi","Jeudi","Vendredi","Samedi");
  	$TI_Mois = array (1 => "Janvier","Février","Mars","Avril","Mai","Juin","Juillet","Août","Septembre","Octobre","Novembre","Décembre");

  	$s_NewDate = "" ;
  	if( $s_Jour < 10 ) $s_Jour = '0'.$s_Jour ;
  	if( $s_Mois < 10) $s_Mois = '0'.$s_Mois ;
  	switch ($pFormat)
  	{
    	case 1: $s_NewDate = "$s_Jour.$s_Mois.$s_Annee4" ; break ;
    	case 2: $s_NewDate = "$s_Jour/$s_Mois/$s_Annee4" ; break ;  // default format
    	case 3: $s_NewDate = "$s_Jour$s_Mois$s_Annee2" ; break ;
    	case 4: $s_NewDate = "{$TI_JourSemaine[$i_JourSemaine]} $s_Jour {$TI_Mois[$s_Mois]} $s_Annee4" ; break;
    	case 5: $s_NewDate = "{$TI_JourSemaine[$i_JourSemaine]} $s_Jour {$TI_Mois[$s_Mois]}" ; break;
    	case 6: $s_NewDate = "$s_Jour {$TI_Mois[$s_Mois]}" ; break ;
    	case 7: $s_NewDate = "$s_Jour/$s_Mois/$s_Annee2" ; break ;
  	}
  	return $s_NewDate ;	
 }
 
 /**
  * 
  * Formate une date au format $format en une date Mysql  yyyy-mm-dd
  * Utilise dd, mm et yyyy ou yy pour spécifier le format d'entrée
  *
  * @param string $date
  * @param string $format
  * 
  * @return string  MySQL date yyyy-mm-dd
  * 
  */
 public function _formatToMysql($pStringDate , $pFormat="ddmmyy")
 {
 		$mysqlDate = null;
 		$dPos = strpos($pFormat,'dd');
 		$mPos = strpos($pFormat,'mm');
 		// ajout jour et mois si dispo 
		if( $dPos !== false and $mPos !== false )
		{
			$d = substr($pStringDate,$dPos,2);
 			$m = substr($pStringDate,$mPos,2);
 			$mysqlDate = "-$m-$d";
		}
		// ajout de l'année
		$y = '' ;
 		$yPos = strpos($pFormat,'yyyy');
 		if( $yPos === false ) // pas de yyyyy test yy
 		{
 			$yPos = strpos($pFormat,'yy');
 			if( $yPos !== false )
 			{
 				$y = substr($pStringDate,$yPos,2);
 				$y += ( $y < 80 )? 2000 : 1900 ; // ne gère que les années de 1980 à 2079 :)
 			}
 		} else {
 			$y = substr($pStringDate,$yPos,4);
 		}
 		if( $y != '' ) $mysqlDate = $y.$mysqlDate;
		return $mysqlDate ;
 }
 
 /**
  * Créé une date PHP a partir d'une chaine au format 
  * $pformat doit utiliser dd mm yy ou yyyy pour identifier
  * le jour (dd), le mois (mm) et l'année (yy ou yyyy)
  *
  * @param unknown_type $pStringDate
  */
 public function createDate($pStringDate,$pFormat='ddmmyy')  //  
 {
 		$dPos = strpos($pFormat,'dd');
 		$d = substr($pStringDate,$dPos,2);
 		$mPos = strpos($pFormat,'mm');
 		$m = substr($pStringDate,$mPos,2);
 		$yPos = strpos($pStringDate,'yyyy');
 		if( $yPos === false ) // pas de yyyyy test yy
 		{
 			$yPos = strpos($pFormat,'yy');
 			$y = substr($pStringDate,$yPos,2);
 			$y += ( $y < 80 )? 2000 : 1900 ; // only year from 1980 to 2079 :)
 		} else {
 			$y = substr($pStringDate,$yPos,4);
 		}
 		return mktime(null,null,null,$m-1,$d,$y);
 }
 
/**
	 * retourne 2 date MYSQL par rapport 
	 * a une période et éventuellement 2 dates au format ddmmyy
	 * 
	 * @param string $pPeriode
	 * @param string $debut
	 * @param string $fin
	 */
	public function calculBorneDate($periode,$debut=null,$fin=null)
	{
		//echo "$periode : $debut - $fin";
		switch( $periode )
		{
			case 'dates' : {
				  //$debut = $this->_formatToMysql($debut,'dd/mm/yyyy');
				  //$fin = $this->_formatToMysql($fin,'dd/mm/yyyy');
				  break;
        }
        case 'yearMonth' : {
           // calcul 2 date par rapport à une plage de mois   $debut et $fin = YYYYMM
           $yearDebut = substr($debut,0,4);
           $monthDebut = substr($debut,4,2);
           $yearFin = substr($fin,0,4);
           $monthFin = substr($fin,4,2);
           $debut = date('Y-m-d', mktime(0,0,0,$monthDebut,1,$yearDebut)); 
           $fin = date('Y-m-d', mktime(0,0,0,$monthFin+1,1,$yearFin)-84600); 
           break;
        }
			default : 
				{ 
					$borneDate = $this->getDebutFinForPeriode($periode);
					$debut = $borneDate['debut'];
					$fin = $borneDate['fin'];
				}	
		}
		//echo "$debut - $fin";
		return array('debut'=>$debut,'fin'=>$fin);
	}
	
 
 /* calcul une date debit et une date fin pour une période
  * en fonction de la date du jour
  * 
  * date anneeEncours, anneePrecedente, 
  * moisEnCours, moisPrecedent, moisSuivant, 
  * trimestreEnCours, trimestrePrecedent
  * tout
  *  
  */
 public function getDebutFinForPeriode($periode)
 {		
 		$d = new DateTime();
 		$mois = $d->format('m');
 		$annee = $d->format('Y');
 		switch($periode)
 		{
 			
 			case 'moisEnCours': { 
 					$debut = "$annee-$mois-01";
 					$fin = date('Y-m-d', mktime(0,0,0,$mois+1,1,$annee)-84600); 
 					break;
 			}
 			
 			case 'trimestreEnCours': {
 				if( $mois > 9 ) // 4 trim
 				{
 					$debut = "$annee-10-01";
 					$fin = "$annee-12-31";
 				} elseif ( $mois > 6 ) // 3 trim 
 				{
 					$debut = "$annee-07-01";
 					$fin = "$annee-09-30";
 				} elseif ($mois>3) //  2 trim 
 				{
 					$debut = "$annee-04-01";
 					$fin = "$annee-06-30";		
 				} else {
 					$debut = "$annee-01-01";
 					$fin = "$annee-03-31";
 				}
 				break;
 			}
 			
 			case 'semestreEnCours' : {
 				if( $mois>6) // 2 semestre
 				{
 					$debut = "$annee-07-01";
 					$fin = "$annee-12-31";
 				} else { // 1 semestre
 					$debut = "$annee-01-01";
 					$fin = "$annee-06-30";
 				}
 				break;
 			}
 			
 			case 'anneeEnCours': $debut = "$annee-01-01"; $fin="$annee-12-31"; break;
 			
 			case 'moisPrecedent': { 
 			 			$mois-- ; 
 			 			$debut = date('Y-m-d',mktime(0,0,0,$mois,1,$annee));
 						$fin = date('Y-m-d', mktime(0,0,0,$mois+1,1,$annee)-84600);  
 			 			break;
 			}
 			
 			case 'trimestrePrecedent': {
 				if( $mois > 9) // 4 trim => 3 trim
 				{
 					$debut = "$annee-07-01";
 					$fin = "$annee-09-30";
 				} elseif( $mois > 6 ) // 3 trim => 2 
 				{
 					$debut = "$annee-04-01";
 					$fin = "$annee-06-30";
 				} elseif ( $mois > 3 ) // 2 trim => 1
 				{
 					$debut = "$annee-01-01";
 					$fin = "$annee-03-31";
 				} else //  1 trim => 4 trim annee-1 
 				{
 					$annee--;
 					$debut = "$annee-10-01";
 					$fin = "$annee-12-31";		
 				} 
 				break;
 			}
 			
 			case 'semestrePrecedent':{
 				if($mois>6) // 2 semestre => 1 
 				{
 					$debut = "$annee-01-01";
 					$fin = "$annee-06-30";
 				} else { // 1 seme => 2 seme annee-1
 					$annee--;
 					$debut = "$annee-07-01";
 					$fin = "$annee-12-31";
 				}
 				break;
 			}
 			
 			case 'anneePrecedente': {
 				$annee-- ; 
 				$debut = "$annee-01-01"; 
 				$fin="$annee-12-31"; 
 				break;
 			}
 			
 		}
      //d(array('debut'=>$debut,'fin'=>$fin));
 		return array('debut'=>$debut,'fin'=>$fin);
 }
 
   /** Ajoute un nombre de jour/mois à une date php */
   public function add($d, $type,$value)
   {
    
   }
 
}

