<?php
/*
Copyright (C) 2013 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

###################################################################
/* 
 * 21/10/14 - Refonte class abstraction BDD Model + dbfunc + dbquery + dbobject
 * 21/10/14 - Utilisation de l'api mysqli
 * 22/10/14 - __construct(), _getColumns(), _truncate(), _queryWith(), _query()
 *						_tableList(), _delete(), _insert(), 
 * 23/10/14 - modif constante SQL_ ajout prefix fonction du driver de BDD dbMysql_
 * 
 * 
 * 
 * @TODO voir si compatible avec API Mysql et PDO
 */

// SQL templates
define('dbMysql_SQL_INSERT','INSERT INTO {TABLE} ({COLS}) VALUES({VALUES}) ') ; 			  
define('dbMysql_SQL_UPDATE','UPDATE {TABLE} SET {VALUES} WHERE {WHERE} {OPTION} ') ; 	
define('dbMysql_SQL_DELETE','DELETE FROM {TABLE} WHERE {WHERE} {OPTION} ');

define('dbMysql_SQL_FIND','SELECT {SELECT} FROM {TABLE} WHERE {WHERE} {OPTION} ');

define('dbMysql_SQL_COUNT','SELECT COUNT(*) AS nbLines FROM {TABLE} WHERE {WHERE} {OPTION}');
define('dbMysql_SQL_SUM','SELECT SUM({COL}) AS total FROM {TABLE} WHERE {WHERE} {OPTION}');

define('dbMysql_SQL_TABLE_LIST','SHOW TABLES');

define('dbMysql_SQL_TRUNCATE','TRUNCATE TABLE {TABLE}'); // !! ne fonctionne pas sur sqllite !!
define('dbMysql_SQL_GET_COLUMNS','SHOW COLUMNS FROM {TABLE}') ;	

define ('MyMVC_DBMYSQL_VERSION','0.3'); // 24/10/14


class MyMVC_dbMysql  {	
	private $_cnx					= null; // id de connexion au SGBD

	public $_error				= '' ;	 // en cas d'erreur contient le libellé de l'erreur
	public $_nbLines			= 0 ; 	 // nombre de lignes affectées par les requetes delete, update, ou nb ligne trouvée pour un select
	public $_lastQuery		= '' ;	 // dernière requete exécutée
	public $_lastInsertId	= null ; // id du dernier enregistrement créé
	
	public function __construct($pConfigDb) {
		if( !empty($pConfigDb['host'])   // controle des parametres
				AND !empty($pConfigDb['user']) 
				AND !empty($pConfigDb['password'])
				AND !empty($pConfigDb['db'])) {
			$this->_cnx = new mysqli( $pConfigDb['host'], $pConfigDb['user'], $pConfigDb['password'], $pConfigDb['db'] ) ;
			if( ! $this->_cnx ) {
				throw new Exception(MyMVC::getTrad('mymvc_db_connexionBaseDeDonneesImpossible'));
			}
		} else {
			throw new Exception(MyMVC::getTrad('mymvc_db_parametreConnexionBaseDeDonneesManquant'));
		}
	}
	
	/** 
	 * Exécute une requete SQL (fonction principale)
	 * 
	 * @param type $pSql   requete SQL
	 * @param type $pVars  variables
	 * @param type $pType  type de requetes 0 = SELECT, 1 = INSERT, 2 = DELETE ou autre
	 * @param type $pLabel Nom col pour récuperer une valeur directement
	 * @return array if ok, null if no data, false if error
	 * 
	 */
	public function _query($pSql , $pType = 0, $pCols = ''  ) {
		MyMVC::_getInstance()->_log(MYMVC_INFO,$pSql);
		if(empty($pSql)) {
			MyMVC_ERROR::_getInstance()->_addError(MyMVC::getTrad('mymvc_db_erreurRequeteSqlManquante'));
			return false;
			//throw new Exception(MyMVC::getTrad('mymvc_DbQueryQueryParametreIncorrect'));
		}
		$this->_nbLines			=	0 ;
		$this->_error				=	'' ;
		$this->_lastQuery		=	'' ;
		$this->_currentId		=	-1 ;
		$recordSet					=	null ;
		
		// remplace les xxx=yyy par les nom et valeurs dans $pCols
		if( is_array($pCols) AND count($pCols)>0 ) {
			foreach( $pCols as $ref=>$val) {
				$pSql = str_replace( "$ref", "$val" , $pSql ) ;  // on remplace dans R_Sql tous les champs refs par leurs valeurs
			}
		}
		
		// exec de la requete
		$rs = $this->_cnx->query( $pSql );
		MyMVC::_getInstance()->_log(MYMVC_SQL,$pSql,MYMVC_LOG);
		if( !$rs ) {
			MyMVC_ERROR::_getInstance()->_addError(MyMVC::getTrad('mymvc_db_erreurRequeteSQL',array( $this->_cnx->errno, $this->_cnx->error)));
			return false;
			//throw new Exception(MyMVC::getTrad('mymvc_db_erreurRequeteSQL', array($this->_cnx->errno,$this->_cnx->error)));
		}
		// recupération du résultat
		$ret = null;
		switch($pType) {
			case '0': { // SELECT
				$this->_nbLines = $rs->num_rows;
				$rs->data_seek(0);
				while($row = $rs->fetch_assoc()) {
					$recordSet[] = $row;
				}
				$ret = $recordSet ;
				break;
			}
			case '1' : { // insert avec autoincremente
				$this->_lastInsertId = mysqli_insert_id($this->_cnx);
				$this->_nbLines = mysqli_affected_rows($this->_cnx) ;
				if( $this->_nbLines == 1) {
					$ret =  true ;
				} else {
					MyMVC_ERROR::_getInstance()->_addError(MyMVC::getTrad('mymvc_db_erreurRequeteInsert'));
					$ret =  false;
				}
				break;
			}
			case '2' : { // insert normal, update, delete ...
				$this->_nbLines = mysqli_affected_rows($this->_cnx) ;
				$ret =  true ;
				break;
			}
			default: {
				MyMVC_ERROR::_getInstance()->_addError(MyMVC::getTrad('mymvc_db_erreurTypeRequete'));
				$ret =  false;
				//throw new Exception(MyMVC::getTrad('mymvc_db_erreurTypeRequete'));
			}
		}
		$this->_lastQuery = $pSql;
		return $ret ;
	} //_query

	
	/** 
	 * Retourne la liste des tables de la connexion en cours
	 */
	public function _tableList() {
		$tableList = array();
		$res = $this->_query(dbMysql_SQL_TABLE_LIST); // requete select
		foreach($res as $one) {
			$tmp = array_values($one);
			$tableList[] = $tmp[0];
		}
		return $tableList;
	} // _tableList
	
	/** 
	 * Retourne la liste des colonnes d'une table
	 */
	public function _getColumns($pTable) {
		if(empty($pTable)) {
			throw new Exception(MyMVC::getTrad('mymvc_db_getColumnsNomDeTableManquant'));
		} else {
			$columns = array();
			$sql = str_replace('{TABLE}',$pTable,dbMysql_SQL_GET_COLUMNS);
			$res = $this->_query($sql);
			foreach($res as $one) {
				$columns[] = $one['Field'];
			}
		}
		return $columns;
	} // _getColumns
	
	
	/** 
	 * Supprime une ligne dans la table
	 */
	public function _delete($pTable=null , $pPrimaryKey=null ,$pVal=null ) {
		if(	empty($pTable) OR empty($pPrimaryKey) OR empty($pVal) ) {
			throw new Exception(MyMVC::getTrad('mymvc_db_DeleteParametreIncorrect'));
		}
		$sql = str_replace('{TABLE}', $pTable , dbMysql_SQL_DELETE ) ;
		$sql = str_replace('{WHERE}', "$pPrimaryKey='$pVal'", $sql);
		$sql = str_replace( '{OPTION}' , ' limit 1' , $sql ) ;
		return $this->_query($sql,2); // requete delete 
	} // _delete
	
	
	/*
   * update un enregistrement dans la base
   * $pData contient des nom/valeurs pour les colonnes de la table
   * les valeurs ont été validées et formatées au préalable
	 * 
   * 
   */
  public function _update($pTable=null, $pData=null, $pPkName=null, $pPkVal=null) {
   	if( empty($pTable) OR !is_array($pData) OR count($pData)==0 ) {
			MyMVC_ERROR::_getInstance()->_addError(MyMVC::getTrad('mymvc_db_updateParametreIncorrect'));
   		return false;
   	}
		if( empty($pPkName) OR empty($pPkVal) ) {
			MyMVC_ERROR::_getInstance()->_addError(MyMVC::getTrad('mymvc_db_updateInfoPkManquante'));
   		return false;
		}
   	$values= '' ;
   	foreach($pData as $n=>$v) {
   		if(!get_magic_quotes_gpc()) $v = addslashes($v); // protection des donnnées si magic_quote OFF
   		$values .= ($values=='')? "$n='$v'" : ", $n='$v'" ; // construit la liste values('val1','val2'..)
   	}

   	$sql = str_replace( '{TABLE}' , $pTable , dbMysql_SQL_UPDATE ) ;
   	$sql = str_replace( '{VALUES}' , $values , $sql ) ;
   	$sql = str_replace( '{OPTION}' , ' LIMIT 1' , $sql ) ; // securité !
   	$sql = str_replace ( '{WHERE}' , " $pPkName='$pPkVal' ", $sql);
   	return $this->_query($sql,2) ; // requete update
  } // _update
	
	
	/** 
	 * Insert créé un nouvel enregistrement dans la BDD 
	 * 
	 */
	public function _insert( $pTable=null, $pData=null ) {
		if( empty($pTable) OR !is_array($pData) OR count($pData)==0 ) {
			throw new Exception(MyMVC::getTrad('mymvc_db_InsertParametreIncorrect'));
		}
		$cols		=	''	;
		$values	=	''	;
		foreach( $pData as $c=>$v ) {
			if( !get_magic_quotes_gpc() ) { $v = addslashes($v); } // protection des donnnées si magic_quote OFF   
			$cols .= ($cols=='')? "$c" : ", $c" ; // construit la liste set(col1,col2..)
			$values .= ($values=='')? "'$v'" : ", '$v'" ; // construit la liste values('val1','val2'..)
		}
		$sql = str_replace( '{TABLE}' , $pTable , dbMysql_SQL_INSERT) ;
		$sql = str_replace( '{COLS}' , $cols , $sql ) ;
		$sql = str_replace( '{VALUES}' , $values , $sql ) ;
		return $this->_query($sql,1) ; // requete insert
	} // _insert	
	
	
	/** truncate (vide) une table */
	public function _truncate($pTable=null) {
		if( empty($pTable) ) {
			throw new Exception(MyMVC::getTrad('mymvc_db_TruncateTableParametreIncorrect'));
		}
		$sql = str_replace('{TABLE}', $pTable, dbMysql_SQL_TRUNCATE) ;
		return $this->_query($sql,2); // req delete
  } // _truncateTable
	
	/** 
	 * Execute une requete SQL
	 * les éléments SQL sont fournis dans le tableau $pSqlParts
	 * 
	 * SELECT, FROM, WHERE, ORDER, OFFSET, LIMIT, GROUPBY
	 * 
	 * @param array $pSqlParts
	 * @return type
	 */
	public function _queryWith($pSqlParts) {
		if( empty($pSqlParts['SELECT']) OR empty($pSqlParts['FROM']) ) {
			MyMVC_ERROR::_getInstance()->_addError(MyMVC::getTrad('mymvc_Model_makeQueryParametreIncorrectPourConstruireSelect'));
			$ret = false;
			//throw new Exception(MyMVC::getTrad('mymvc_Model_makeQueryParametreIncorrectPourConstruireSelect'));
		} else {
			$sql = 'SELECT '.$pSqlParts['SELECT'].' FROM '.$pSqlParts['FROM'];
			if(!empty($pSqlParts['WHERE'])) {		$sql .= ' WHERE '.$pSqlParts['WHERE']; }		
			if(!empty($pSqlParts['GROUPBY'])) {	$sql .= ' GROUP BY '.$pSqlParts['GROUPBY'];	}
			if(!empty($pSqlParts['ORDER'])) {		$sql .= ' ORDER BY '.$pSqlParts['ORDER'];	}
			if(!empty($pSqlParts['LIMIT'])) {		$sql .= ' LIMIT '.$pSqlParts['LIMIT']; }
			if(!empty($pSqlParts['OFFSET'])) {		$sql .= ' OFFSET '.$pSqlParts['OFFSET']; }
			$ret = $this->_query($sql,0);
		}	
		return $ret;
	} // _queryWith
	
}