<?php
/*
Copyright (C) 2011 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

#################################################################

 * 20/10/14 - Refonte class model pour simplifier les appels et optimiser l'occupation mémoire
 *						en supprimant l'utilisation de la classe dbObject, dbQuery et dbFunc
 *						Schema de fonctionnement
 *						MyMVC_Model => MyMVC_dbMysql ou MyMVC_dbSqlite ou MyMVC_dbPostgres...
 *						MyMVC_Model fournit l'abstraction entre les object red et les appel a la DB
 * 21/10/14 - __construct(), _create(), _save(), _delete();
 * 22/10/14 - _findAll(), _truncateTable(), _setOrdre(), _setLimit(), _setOffset(), _setGroupby, _setSelect, _setFrom
 *						_find(),
 * 27/10/14 - _sum(), _count(), _mix(), _max(), _deleteBySql(), _deleteByPk(), _truncateTable(), _findAllIndexed(), _getColValue()
 * 
 * 
 * 
 * @TODO 
 * 
 * 
 * 
 */

define ('MyMVC_MODEL_VERSION','2.0');

Class MyMVC_Model {

	public $_aspect					= 'form';   // aspect par défaut pour le afterRead()
	public $_applyAfterRead			= true;
	public $_applyBeforeSave		= true;
	public $_tableName				= null ;
	public $_sqlCreateTable			= '' ;      // code SQL pour créer la table si besoin
	
	private $_dbConnect				= null ;
	private $_name					= null ;
	private $_pkName				= null ;
	private $_pkAutoIncrement		= true ;
	private $_pkGenFunc				= null ;	// func to create a key when not autoincremented
	private $_tableStruct			= null ;
	private $_serializedCols		= array() ;	// format cols before write or after read
	private $_defaultValues			= array() ;
	private $_specialCols			= array() ;
	private $_SELECT				= null ;	// éléments pour la requete SQL
	private $_FROM					= null ;
	private $_WHERE					= null ;
	private $_LIMIT					= null ;
	private $_OFFSET				= null ;
	private $_ORDER					= null ;
	private $_GROUPBY				= null ;

	/**
	 * 
	 * @param type $pData
	 */
	public function __construct($pName , $pUsePrefix = true) {
		if( empty($pName) ) {
			throw new Exception(MyMVC::getTrad('mymvc_Model_constructNomModelManquantPour'));
		}
		$this->_name = $pName;
		$dbConfig = MyMVC_Config::_getInstance()->dbConfig ;
		// controle s'il faut utiliser un prefix
		if($pUsePrefix) { 
			$this->_tableName = $dbConfig['prefix'] . $pName ;
		} else {
			$this->_tableName = $pName ;
		}
		// chargement du driver BDD
		switch(MyMVC_Config::_getInstance()->dbConfig['driver']) {
			case 'mysql': {
				require_once('lib/dbMysql.class.php'); 
				$this->_dbConnect = new MyMVC_dbMysql($dbConfig);
				break;
			}
			case 'sqlite': { 
				require_once('lib/dbSqlite.class.php');
				$this->_dbConnect = new MyMVC_dbSqlite($dbConfig);
				break;
			}
			default: throw new Exception(MyMVC::getTrad('mysql_ModelDriverDatabaseIncorrect',MyMVC_Config::_getInstance()->dbConfig['driver']));
		}
		unset($dbConfig);
		// recupere la liste des table
		$tableExist = false;
		if( !is_array(MyMVC::_getInstance()->_getValue('tableList')) OR MyMVC::_getInstance()->_getValue('tableList') == 0 ) {
			MyMVC::_getInstance()->_storeValue('tableList',$this->_dbConnect->_tableList());
		}
		// controle si la table existe
		foreach(MyMVC::_getInstance()->_getValue('tableList') as $one) {
			if( $one == $this->_tableName) { 
				$tableExist = true ;
			}
		}
		// Creation table si besoin
		if( !$tableExist ) { 
			MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Model::__createTable() '.$this->_tableName,MYMVC_LOG);
			if( $this->_sqlCreateTable != '' ) {
				// @TODO vérifier que CREATE TABLE est compatible avec sqlite, postgres etc...
				$this->_dbConnect->_query('CREATE TABLE '.$this->_tableName.' '.$this->_sqlCreateTable,2);
				// chargement des données dans la table
				$this->initTable(); 
				// mise à jour de la liste des tables
				MyMVC::_getInstance()->_storeValue('tableList',$this->_dbConnect->_tableList());
			} else {
				throw new Exception(MyMVC::getTrad('sqlCreateTableManquant',$this->_name));
			}
		} 
		// Valeur par defaut pour SQL
		$this->_SELECT = $this->_tableName.".*" ;
		$this->_FROM = $this->_tableName ;
		$this->_tableStruct = $this->_dbConnect->_getColumns($this->_tableName);
	}
	
	/** filtre les appels sur des methodes inexistantes */
	public function __call($pFunc,$pParams) { 
		throw new Exception(MyMVC::getTrad('mymvc_Model_methodeModelInconnue',array($pFunc)));
	}
	
	/**
	* Set primary key, 
	* $pAutoIncrement = true indique que la PK est autoincrementée
	* $pPkFunc indique le nom de fonction pour générer la PK, $pAutoIncrement doit être à false
	*
	* @param string $pKey
	* @param bool $pAutoIncrement
	* @param string $pPkFunc
	 * 
	*/
	public function _setPrimaryKey($pKey, $pAutoIncrement = true, $pPkFunc = null ) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Model::_setPrimaryKey()',MYMVC_LOG);
		if(empty($pKey)) {
			throw new Exception(MyMVC::getTrad ('mymvc_Model_setPrimaryKey_nomClePrimaireIncorrect'));
		}
		$this->_pkName = $pKey ;
		$this->_pkAutoIncrement = $pAutoIncrement ;
		if( !$pAutoIncrement ) {
			if( empty($pPkFunc) ) {
				throw new Exception(MyMVC::getTrad ('mymvc_Model_setPrimaryKey_fonctionCreationClePrimaireManquante'));
			}
			if( !method_exists($this,$pPkFunc) ) { // fonction PK manquante
				throw new Exception(MyMVC::getTrad('mymvc_Model_insertFunctionCreatePkmanquante',array($pPkFunc)));
			}
			$this->_pkGenFunc = $pPkFunc ;
		}
	}
	
	
	/** 
	 * Creer un array avec toute les colonnes de la table 
	 * calcul les valeurs par défaut si besoin
	 * ainsi que les specialCols
	 * si $pCols!=null les valeurs sont utilisé pour remplir l'array
	 * @param type $pCols
	 */
	public function _create($pCols=null) {
		$dbObject = array();
		// valeur par défaut
		if( count($this->_defaultValues)>0 ) {
			//echo 'dfv';
			foreach($this->_defaultValues as $one) {
				$func = $one['func'];
				if( $one['isMethode']) {
					$dbObject[$one['col']] = $this->$func() ; // methode du model
				} else {
					$dbObject[$one['col']] = $func() ; // function
				}
			}
		}
		// affectation des valeurs si besoin
		if( $pCols!=null AND is_array($pCols) AND count($pCols)>0 ) {
			foreach($pCols as $col=>$val) {
				$dbObject[$col] = $val ;
			}
		}
		// ajoute les colonnes manquantes
		foreach($this->_tableStruct as $one) {
			if( !isset($dbObject[$one]) AND $one!=$this->_pkName ) {
				$dbObject[$one] = '';
			}
		}
		return $dbObject;
	}
	
	/** ajoute une fonction pour la valeur par défaut d'une colonne */
	public function _addDefaultValue($pCol,$pFunc,$pIsMethod=true) {
	//public function _setDefaultValueFunc($pCol, $pFunc, $pIsMethode = true ) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Model::_addDefaultValueFunc()',MYMVC_LOG);
		if( empty($pCol) OR empty($pFunc) ) {
	 		throw new Exception(MyMVC::getTrad('mymvc_model_addDefaultValueParametreIncorrect'));
	 	}
	 	$this->_defaultValues[] = array('col'=>$pCol,'func'=>$pFunc,'isMethode'=>$pIsMethod) ;
	}
	
	public function _getNbLines() { return $this->_dbConnect->_nbLines; }
	public function _getLastQuery() { return $this->_dbConnect->_lastQuery; }
	public function _getLastInsertedId() { return $this->_dbConnect->_lastInsertedId ; }
	
	/** 
	 * Exécute une requête SQL directement 
	 * seul les requetes INSERT et UPDATE ne sont pas prises en compte
	 * 
	 * @param type $pSql
	 * @return boolean
	 * 
	 */
	public function _directQuery($pSql) {
		if(empty($pSql)) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_directQueryParametreIncorrect'));
			return false;
		}
		// @TODO identifier la requete SELECT
	 $ret = $this->_dbConnect->_query($pSql);
	 return $ret;
	}
	
	/** enregistre une ligne dans une table 
	 * si $pkVal == null c'est un insert, sinon un update
	 * 
	 * Avant d'enregistrer les appels suivant sont effectués
	 * beforeSave($data) // controle si true on continue
	 * isValid($data)  // le test doit porter sur les valeur formatée pour la BDD
	 * formatForDb($data) // formatage des cols pour la BDD
	 * 
	 * Après enregistrement
	 * afterSave($data) // opération a effectuer apres enregistrement
	 * 
	 * 	 */
	public function _insert($pData) {
		if( empty($pData) OR !is_array($pData) OR count($pData)==0) {
			throw new Exception(MyMVC::getTrad('mymvc_Model_insertParametreIncorrect'));
		} 
		// beforeSave()
		if( ! $this->beforeSave($pData) ) {
			return false;
		}
		// isValid()
		$isValid = true;
		foreach($pData as $col=>$val) {
			$err = $this->isValid($col,$pData) ;
			if( $err != null ) {
				MyMVC_Error::_getInstance ()->_addError($err);
				$isValid = false;
			}
		}
		if(!$isValid) { 
			return false;
		}
		// formatage spécifique pour la BDD
		$pData = $this->formatToDb($pData);
		// serialisation de valeur
		foreach($this->_serializedCols as $one) {
			if( !empty($pData[$one])) {
				$pData[$one] = serialize($pData[$one]);
			}
		}
		//echo $this->_pkName ;
		//echo $this->_pkAutoIncrement ;
		if( $this->_pkName === null OR $this->_pkAutoIncrement === true ) { // pas de PK ou PK en autoIncremente
			$ret = $this->_dbConnect->_insert($this->_tableName , $pData ); // insert avec autoincremente
		} else {
			if(empty($this->_pkGenFunc)) { // fonction PK non définit
				$this->_addError(MyMVC::getTrad('mymvc_Model_insertFunctionCreatePkVide',array($funcPk)));
				return false;
			}
			if( empty($pData[$this->_pkName]) ) { // possibilité de forcer la PK en entrée
				$funcPk = $this->_pkGenFunc ;
				if( !method_exists($this,$funcPk) ) {
					$this->_addError(MyMVC::getTrad('mymvc_Model_insertFunctionCreatePkManquante',array($funcPk)));
					return false;
				}
				$pData[$this->_pkName] = $this->$funcPk(); // creation PK
			}
			$ret = $this->_dbConnect->_insert($this->_tableName , $pData ); // insert avec pk générée
		}
		return $ret;
	}
	
	
	/** 
	 * Mise à jour d'une ligne dans une table 
	 * $pPkVal = valeur de la PK pour la mise à jour
	 * 
	 * Avant d'enregistrer les appels suivant sont effectués
	 * beforeSave($data) // controle si true on continue
	 * isValid($data)  // le test doit porter sur les valeur formatée pour la BDD
	 * formatForDb($data) // formatage des cols pour la BDD
	 * 
	 * Après enregistrement
	 * afterSave($data) // opération a effectuer apres enregistrement
	 * 
	 * @TODO _update sans clé primaire ???
	 * 
	 */
	public function _update($pData) {
		if( empty($pData) OR !is_array($pData) OR count($pData)==0 OR empty($pData[$this->_pkName]) ) {
			throw new Exception(MyMVC::getTrad('mymvc_Model_updateParametreIncorrect'));
		} 
		$pkVal = $pData[$this->_pkName]; // recup clé primaire
		unset( $pData[$this->_pkName]); // supppression de la pk des datas
		// beforeSave()
		if( ! $this->beforeSave($pData) ) {
			return false;
		}
		// isValid()
		$isValid = true;
		foreach($pData as $col=>$val) {
			$err = $this->isValid($col,$pData) ;
			if( $err != null ) {
				MyMVC_Error::_getInstance ()->_addError($err);
				$isValid = false;
			}
		}
		if(!$isValid) { 
			return false;
		}
		// formatage spécifique pour la BDD
		$pData = $this->formatToDb($pData);
		// serialisation de valeur
		foreach($this->_serializedCols as $one) {
			if( !empty($pData[$one])) {
				$pData[$one] = serialize($pData[$one]);
			}
		}		
		if( $this->_pkName === null  ) { // pas de PK  = erreur
			$this->_addError(MyMVC::getTrad('mymvc_Model_updatePkNonDefini',array($funcPk)));
			return false;
		}
		return $this->_dbConnect->_update($this->_tableName , $pData, $this->_pkName, $pkVal ); // insert avec pk générée
	} // _update
	
	
	/* retourne la dernière erreur */
	public function _getLastError() { return MyMVC_ERROR::_getInstance()->_getLastError(); }
	/* retourne un tableau avec toutes les erreurs */
	public function _getErrors() { return MyMVC_ERROR::_getInstance()->_getErrors(); }
	/* ajoute une erreur */
	public function _addError($pError) { MyMVC_ERROR::_getInstance()->_addError($pError); }
	
	/** supprime une ligne dans un table */
	public function _delete($pId) {
		if( empty($pId) ) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_deleteParametreIncorrect'));
			return false;
		}
		
		if( !$this->isDeletable($pId) ) {
			return false;
		}
		//echo $pId;
		$this->_dbConnect->_delete($this->_tableName,$this->_pkName,$pId);
		if( $this->_dbConnect->_nbLines != 1) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_deleteErreurExecutionSql',array($this->_dbConnect->_lastQuery)));
			return false;
		}
		return true;
	} // _delete

	/** Supprime plusieurs lignes dans une tabl
	 * !!!! les controles isDeletable() ne sont pas effectué
	 * 
	 * 
	 * 
	 * 	 */
	public function _deleteBySql($pWhere) {
		if(empty($pWhere) ) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_deleteBySql_parametreIncorrect'));
			return false;
		}
		$sql = "DELETE FROM ".$this->_tableName." WHERE $pWhere";		
		if( !$this->_dbConnect->_query($sql,2) ) {
				//$this->_addError(MyMVC::getTrad('mymvc_Model_getColValue_resultatErrone'));
				$ret = false;
		} else {
			$ret = $this->_getNbLines() ; // nb lines supprimer
		}
		return $ret;
	}

	/**
	 * Supprimer plusieurs enregistrement
	 * mais ici les controles isValid(), beforeDelete() et afterDelete() sont effectués
	 * 
	 * 
	 * @param array $pIds  liste des Pks a supprimer 
	 */
	public function _deleteByPk($pIds) {
		if(empty($pIds) OR !is_array($pIds)) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_deleteCollection_parametreIncorrect'));
			return false;
		}
		$ret=0;
		foreach($pIds as $one) {
			//echo $one;
			if( !$this->_delete($one) ) {
				//echo '--'.$this->_getLastError();
				$ret = false; // sortie si une erreur survient
				break;
			}
			$ret++;
			//echo $ret;
		}
		return $ret;
	}
	
	/** truncate (vide) la table */
	public function _truncateTable() { 
		return $this->_dbConnect->_truncate($this->_tableName);
	}
	
	/** mise à jour de plusieurs lignes dans une table */
	public function _updateCollection($where,$pData) {}

	/** recherche une ligne d'apres la valeur de la PK */
	public function _find($pId) {
		if(empty($pId)) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_findParametreIncorrect'));
			return false;
		}
		// construction requete SQL
		$result = $this->_dbConnect->_queryWith( array(
			'SELECT'=>$this->_SELECT,
			'FROM'=>$this->_FROM,
			'WHERE'=>$this->_pkName."='$pId' "
		));

		if( is_array($result) AND count($result)==1 ) {
			$ret = $result[0];
		} else {
			$ret = $result;
		}
		return $ret;
	}

	/** recherche plusieurs lignes correspondant a une condition */
	public function _findAll($pWhere = null) {
		if(empty($pWhere)) $pWhere = 1 ; // on prend tout
		return $this->_dbConnect->_queryWith( array(
			'SELECT'=>$this->_SELECT,
			'FROM'=>$this->_FROM,
			'WHERE'=>$pWhere,
			'ORDER'=>$this->_ORDER,
			'OFFSET'=>$this->_OFFSET,
			'LIMIT'=>$this->_LIMIT,
			'GROUPBY'=>$this->_GROUPBY
		));
	}

	/** idem _findAll() mais le resultat est convertit en tableau indexé avec la PK 
	 * 
	 * @TODO verifier que l'on dispose bien d'une PK	 * 
	 * 
	 */
	public function _findAllIndexed($pWhere=null) {
		$res = $this->_findAll($pWhere);
		if($res AND count($res)>0) {
			$resIndexed = array();
			foreach($res as $one) {
				$id = $one[$this->_pkName];
				unset($one[$this->_pkName]);
				$resIndexed[$id] = $one;
			}
			$res = $resIndexed;
		}
		return $res;
	}

	/** recherche la valeur d'une colonne spéficique */
	public function _getColValue($pCol,$pWhere) {
		if(empty($pCol) OR empty($pWhere) ) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_getColValue_parametreIncorrect'));
			return false;
		}
		$res = $this->_dbConnect->_queryWith( array(
			'SELECT'=>$pCol,
			'FROM'=>$this->_FROM,
			'WHERE'=>$pWhere)
		);
		if(!is_array($res) OR count($res)!=1) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_getColValue_resultatErrone'));
			$colVal = false;
		} else {
			$rec = $res[0];
			$colVal = $rec[$pCol];
		}
		return $colVal;
	}

	/* effectue un count sur une colonne */
	public function _count($pCol,$pWhere=null) {
		if(empty($pCol)) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_count_parametreIncorrect'));
			return false;
		}
		if($pWhere!==null) {
			$where = $pWhere;
		} else {
			$where = '1'; // pas de filtre
		}
		$res = $this->_dbConnect->_queryWith( array(
			'SELECT'=>"COUNT($pCol) AS __total__",
			'FROM'=>$this->_FROM,
			'WHERE'=>$where)
		);
		if(!is_array($res) OR count($res)!=1) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_count_resultatErrone'));
			$ret = false;
		} else {
			$ret = $res[0]['__total__'];
		}
		return $ret;
	}

	/** effectue un sum sur un colonne */
	public function _sum($pCol,$pWhere=null) {
		if(empty($pCol)) {
		$this->_addError(MyMVC::getTrad('mymvc_Model_sum_parametreIncorrect'));
			return false;
		}
		if($pWhere!==null) {
			$where = $pWhere;
		} else {
			$where = '1'; // pas de filtre
		}
		$res = $this->_dbConnect->_queryWith( array(
			'SELECT'=>"SUM($pCol) AS __total__",
			'FROM'=>$this->_FROM,
			'WHERE'=>$where)
		);
		if(!is_array($res) OR count($res)!=1) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_sum_resultatErrone'));
			$ret = false;
		} else {
			$ret = $res[0]['__total__'];
		}
		return $ret;
	}

	/** retourne la valeur max d'une colonne */
	public function _max($pCol,$pWhere=null) {
		if(empty($pCol)) {
		$this->_addError(MyMVC::getTrad('mymvc_Model_max_parametreIncorrect'));
			return false;
		}
		if($pWhere!==null) {
			$where = $pWhere;
		} else {
			$where = '1'; // pas de filtre
		}
		$res = $this->_dbConnect->_queryWith( array(
			'SELECT'=>"MAX($pCol) AS __MAX__",
			'FROM'=>$this->_FROM,
			'WHERE'=>$where)
		);
		var_dump($res);
		if(!is_array($res) OR count($res)!=1) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_max_resultatErrone'));
			$ret = false;
		} else {
			$ret = $res[0]['__MAX__'];
		}
		return $ret;
	}

	/** retourne la valeur min d'une colonne */
	public function _min($pCol,$pWhere) {
		if(empty($pCol)) {
		$this->_addError(MyMVC::getTrad('mymvc_Model_min_parametreIncorrect'));
			return false;
		}
		if($pWhere!==null) {
			$where = $pWhere;
		} else {
			$where = '1'; // pas de filtre
		}
		$res = $this->_dbConnect->_queryWith( array(
			'SELECT'=>"MIN($pCol) AS __MIN__",
			'FROM'=>$this->_FROM,
			'WHERE'=>$where)
		);
		if(!is_array($res) OR count($res)!=1) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_min_resultatErrone'));
			$ret = false;
		} else {
			$ret = $res[0]['__MIN__'];
		}
		return $ret;
	}

	/** retourne la valeur min d'une colonne */
	public function _average($pCol,$pWhere) {
		if(empty($pCol)) {
		$this->_addError(MyMVC::getTrad('mymvc_Model_average_parametreIncorrect'));
			return false;
		}
		if($pWhere!==null) {
			$where = $pWhere;
		} else {
			$where = '1'; // pas de filtre
		}
		$res = $this->_dbConnect->_queryWith( array(
			'SELECT'=>"AVG($pCol) AS __MOYENNE__",
			'FROM'=>$this->_FROM,
			'WHERE'=>$where)
		);
		if(!is_array($res) OR count($res)!=1) {
			$this->_addError(MyMVC::getTrad('mymvc_Model_average_resultatErrone'));
			$ret = false;
		} else {
			$ret = $res[0]['__MOYENNE__'];
		}
		return $ret;
	}
	
	/** definit la clause ORDER de la requête SQL */
	public function _setOrder($pOrder=null) { $this->_ORDER = $pOrder; }

	/** definit la clause LIMIT de la requête SQL */
	public function _setLimit($pLimit=null) { $this->_LIMIT = $pLimit;}

	/** definit la clause OFFSET de la requête SQL */
	public function _setOffet($pOffset=null) { $this->_OFFSET = $pOffset; }
	
	/** definit la clause GROUPBY de la requête SQL */
	public function _setGroupBy($pGroupBy=null) { $this->_GROUPBY = $pGroupBy; }
	
	/** definit la clause GROUPBY de la requête SQL */
	public function _setWhere($pWhere=null) { $this->_WHERE = $pWhere; }
	
	// @TODO reset a valeur par défaut FROM et SELECT si fonction reçoit null en entrée
	/** definit la clause GROUPBY de la requête SQL */
	public function _setSelect($pSelect=null) { $this->_SELECT = $pSelect; }
	
	/** definit la clause FROM de la requête SQL */
	public function _setFrom($pFrom=null) { $this->_FROM = $pFrom; }
	
	
	/** retourne la requete SQL qui sera exécutée avec les valeurs de select,from,where,... */
	public function _getSqlQuery() {
		return array(
			'SELECT'=>$this->_SELECT,
			'FROM'=>$this->_FROM,
			'WHERE'=>$this->_WHERE,
			'ORDER'=>$this->_ORDER,
			'OFFSET'=>$this->_OFFSET,
			'LIMIT'=>$this->_LIMIT,
			'GROUPBY'=>$this->_GROUPBY
		);
	}

	/** écrit une ligne de log dans logs/app.log */
	public function _log($pMessage) {
		if(!empty($pMessage)) {
			MyMVC::_getInstance()->_log(MYMVC_INFO,$pMessage);
		}
	}

	/**
	 * Ajout un relation JOIN LEFT entre 2 tables
	 * 
	 * $pFkName = clé étrangère table en cours
	 * $pPkName = clé primaire de $pTable
	 * $pType permet de forcer le JOIN => inner, right, par défaut ce sera left join
	 * si $pFkName == $pPkName, $pPkName peut être omis
	 * 
	 * 	 */
	public function _setHasOne($pTable,$pFkName,$pType=null,$pPkName=null) {
		if(!empty($pTable) OR empty($pPkName) ) {
			throw new Exception(MyMVC::getTrad('mymvc_Model_setHasOneErreurParametre'));
		}		
		$test = new $pTable(); // permet de créer la table si besoin  
		$tableName = $test->_tableName ; // recup du nom avec le prefix
		switch($pType) {
			case 'inner' : $type = "INNER" ; break;
			case 'right' : $type = "RIGHT" ; break;
			default: $type = 'LEFT';
		}
		if( $pPkName == null) { // les 2 colonnes pour la relation ont le même nom
			$this->_FROM .= " $type JOIN $tableName USING ($pFkName)" ;
		} else {
			$this->_FROM .= " $type JOIN $tableName ON {$this->_tableName}.$pFkName = $tableName.$pPkName " ;		
		}
		$this->_SELECT .= ",$tableName.*" ;
	}

	/** definit une colonne comme étant sérialisée dans la table */
	public function _addSerializedCol($pCol) {
		if( !empty($pCol)) {
			$this->_serializedCols[] = $pCol ;
		}
	}
	
	/** 
	 * Definit une col spéciale qui ne fait pas partie de la table mais sera ajouté à chaque résultat
	 * 
	 */
	public function _addSpecialCol($pCol, $pFunc) {
		if(empty($pCol) OR empty($pFunc)) {
			throw new Exception(MyMVC::getTrad('mymvc_Model_addSpecialColErreurParametre'));
		}
		$this->_specialCols[$pCol] = $pFunc ;
	}
	
	// methodes pouvant être surchargées 
	public function formatToDisplay($pData,$pAspect) { return $pData; }
	
	public function beforeSave($pData) { return true ;} // si retour = false le save est annulé
	public function formatToDb($pData) { return $pData; } 
	public function afterSave($pData) { return true; } // 

	public function beforeDelete($pData) { return true; } // si retour = false, le delete est annulé
	public function afterDelete($pData) { return true; } // 

	public function isValid($pData) { return null; } // si retour != null => ajoute une erreur
	public function isDeletable($pPkVal) { return true; } // si retour = true le delete est effectué

	public function initTable() { return true; }

}