<?php
/*
Copyright (C) 2009 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

#################################################################

 * Class  		 	MyMVC_Registry  (skeleton)
 * Création le		16/07/09
 * Copyright		Auguste
 *
 * class used to store objects, params values and pass them
 * between all app classes
 * store can be session or db need a table : mymvc_registry
 *
 * 07/11/09 - first usable version with session storage
 * 09/12/09 - _loadSessionRegistry(), _saveSessionRegistry(), _loadDbRegistry(), _saveDbRegistry()
 * 10/03/10 - new method to store in table  PREFIX_registry, one record for each value
 * 18/03/10 - modif param pour appel _updateBySql()
 * 08/02/11 - fix bug purge db registry
 * 09/02/11 - modif _saveDb() pour faire des insert et ainsi permettre un truncate avant chaque save
 * 09/02/11 - modif __set() pour ne pas insérer en base les valeurs, c'est _saveDb() qui s'en charge
 * 17/12/12 - creation automatique de la table si besoin, serialisation uniquement pour les tableaux
 * 17/02/14 - Ajout utilisation d'un prefix pour le multi-user  _mymvc_registry_prefix
 *          - filtrage de la requête SQL pour ne récupérer que les variables avec le bon prefix
 * 1.0.1
 * 10/06/14 - Ajout log nom des méthodes en mode Debug
 * 26/06/14 - Suppression enregistrement valeur vide en DB
 * 1.0.2
 * 30/06/14 - Modif appel _load directement dans _init(), ajout _log() dans __set() pour tracer les modifs de variables
 *  * 
 * 
 * TODO : sqllite backend
 * 
 * 
*/

define ('MyMVC_REGISTRY_VERSION','1.0.2');

class MyMVC_Registry implements ArrayAccess
{
	private static $_instance 	= NULL;
	private $_data 				= array() ;
	private $_loaded 				= false ;  	// true if registry loaded from storage
	private $_saved 				= false ; 	// true if registry is up to date into storage
	private $_storage 			= '' ; 			// current storage (db, session )
  private $_tableName        =  ''; // nom de la table pour stocker la registry
  private $_mymvc_registry_prefix = '';  // prefix pour les noms des variables, utile pour le multi-user
   
	/**
	 * return instance
	 *
	 * @return MyMVC_Registry instance 
	 * 
	 */
	public static function _getInstance()
	{
    if(self::$_instance == NULL )
    {
			
      self::$_instance = new MyMVC_Registry();
    }
     return self::$_instance;
  }

	/**
	 * contructor
	*/
   function __construct() {
		 MyMVC::_getInstance()->_log(MYMVC_DEBUG,'New Registry object',MYMVC_LOG);
		 self::$_instance = $this ;
	 }
						
   /** init registry */
   public function _init($pStorageType) {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::_init()',MYMVC_LOG);
		// controle si la table registry existe
		if( MyMVC_Config::_getInstance()->enableRegistry == 1 AND MyMVC_Config::_getInstance()->registryStorage == 'db')
		{
			$this->_tableName = MyMVC_Config::_getInstance()->dbConfig['prefix'].'_registry' ;
			$this->__checkTable();
			$this->_load($pStorageType);
			MyMVC::_getInstance()->_log(MYMVC_INFO,"Registry loaded",MYMVC_LOG);
		} else {
			MyMVC::_getInstance()->_log(MYMVC_INFO,"Registry disabled",MYMVC_LOG);
		}
	}
	
   public function _setPrefix($pPrefix) {
		 MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::_setPrefix()',MYMVC_LOG);
     // TODO save et load des valeurs lors du changement de prefix
     $this->_mymvc_registry_prefix = $pPrefix.'_';
   }
   
	 public function _getPrefix() { return $this->_mymvc_registry_prefix; }
	 
   /** Controle si la table de stockage pour la registry existe et la créé si nécessaire */
   private function __checkTable()
   {
		 MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::__checkTable()',MYMVC_LOG);
     //$this->_tableName = MyMVC_Config::_getInstance()->dbConfig['prefix'].'_registry' ;
     $createTable = true;
     $db = new MyMVC_DbQuery(MyMVC_Config::_getInstance()->dbConfig);
     $rs = $db->_query("SHOW TABLES");
     if( count($rs)>0 )
     {
         foreach($rs as $one)
			{
				$v = array_values($one);
				$tableListe[] = strtolower($v[0]);
			}
         if( in_array($this->_tableName, $tableListe) ) $createTable = false;
         //$this->_data['_tableList'] = $tableListe;
     }
     
     if( $createTable)
     {
        $sql = "CREATE TABLE ".$this->_tableName."  (
                  `id` VARCHAR( 30 ) NOT NULL ,
                  `value` TEXT NULL ,
                  `date` DATE NOT NULL,
                  `serialized` TINYINT(1) NOT NULL,
                  UNIQUE (`id`)
               ) ENGINE = MYISAM ;";
        $db->_query($sql);
        MyMVC::_getInstance()->_log(MYMVC_DEBUG,'Création table Registry');
     }
   }
   
  /**
   * save registry into storage
   * 
   * @return unknown_type
   */
  public function _save() {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::_save()',MYMVC_LOG);
		//echo '---'.$this->_saved ;
      if( $this->_storage != null and $this->_saved != true ) { // only save if needed !
         switch($this->_storage) {
            case 'session' : {
               $this->__saveSessionRegistry();
               break;
            }
            case 'db' : { 
               $this->__saveDbRegistry();
               break;
            }
            case 'sqlite' : {
               $this->__saveSqliteRegistry();
               break;
            }
            default: {
               throw new Exception( MyMVC::getTrad('mymvc_RegistryNoStorage',array($this->_storage)));
            }
         }
         MyMVC::_getInstance()->_log(MYMVC_INFO,MyMVC::getTrad('mymvc_RegistrySaved') ,MYMVC_LOG) ;
      } else {
         MyMVC::_getInstance()->_log(MYMVC_INFO,MyMVC::getTrad('mymvc_RegistryNoStorageOrNothingToSave') ,MYMVC_LOG) ;
      }
  }
  
  
  /**
   * 
   * load registry from storage
   * 
   * @return 
   * 
   */
  public function _load($pStorage)
  {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::_load()',MYMVC_LOG);
  	if( $pStorage != '')
  	{
  		$this->_data = array(); //reset values
  		switch( $pStorage)
  		{
  			case 'session' : {
  				$this->__loadSessionRegistry();
  				break;
  			}
  			case 'db' : {
  				if( DBCONFIG !=  '') {
  					$this->__loadDbRegistry();
  					break;
  				} else {
  					throw new Exception('Db storage for regitry need DBCONFIG to be set in site-conf.php');
  					//die('rrr');
  				}
  			}
  			case 'sqlite':
  				$this->__loadSqliteRegistry();
  				break;
  			default: {
  				throw new Exception("Error loading registry, unknown storage : $pStorage");
  			}
  		}
  	} else {
  		throw new Exception("Error loading registry, no storage supplied");
  	}
  }
	
  /** charge la registry depuis une base sqllite */
  private function __loadSqliteRegistry()
  {
  	MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::__loadSqliteRegistry()',MYMVC_LOG);
  }
  
  /** sauve la registry dans une base sqllite */
  private function __saveSqliteRegistry()
  {
  	MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::__saveSqliteRegistry()',MYMVC_LOG);
  }
  
  
  /** charge la registry depuis la session en cours */
  private function __loadSessionRegistry()
  {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::__loadSessionRegistry()',MYMVC_LOG);
  	$this->_data = unserialize($_SESSION['mymvc_registry']) ;
  	$this->_loaded = true ;
  	$this->_saved = true ;
  	$this->_storage = 'session' ;
  }
  
  /** enregistre la registry en session */
  private function __saveSessionRegistry()
  {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::__saveSessionRegistry()',MYMVC_LOG);
  	$_SESSION['mymvc_registry'] = serialize($this->_data) ;
  	$this->_saved = true ;
  }

  /** charge la registry depuis le SGBD */
  private function __loadDbRegistry()
  {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::__loadDbRegistry()',MYMVC_LOG);
   //$this->__checkTable();
  	$this->_storage = 'db' ;
  	$this->_loaded = true ;
  	$this->_saved = true ; 
  	$db = new MyMVC_DbQuery( MyMVC_Config::_getInstance()->dbConfig ) ;
    $filtre = ($this->_mymvc_registry_prefix!='')?  "WHERE id like '".$this->_mymvc_registry_prefix."%'" : '' ;
  	$result = $db->_query( "SELECT * FROM ".$this->_tableName. " $filtre" ) ;
  	if( count($result) > 0 )
  	{
  		foreach($result as $one)
  		{
         if( $one['serialized']==1)
         {
            $this->_data[$one['id']] = unserialize($one['value']) ;
         } else {
            $this->_data[$one['id']] = $one['value'] ;
         }
  		}
  	} else {
  		$this->_clear() ; // empty registry ;
  	}
  }

  /** sauve la registry dans le SGBD */
  private function __saveDbRegistry()
  {
     MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::__saveDbRegistry()',MYMVC_LOG);
     $db = new MyMVC_DbQuery( MyMVC_Config::_getInstance()->dbConfig ) ;
     if($this->_saved == false)
     { 
         $db->_truncateTable($this->_tableName); // purge des données en base
         foreach( $this->_data as $id=>$val)
         {
						if($val!='') { // pas de sauvegarde si valeur null
							if( is_array($val) )
							{
								 $values = array( 'id'=>$id,"value"=>serialize($val) , "date"=>date('Y-m-d'),'serialized'=>1 ) ;
							} else {
								 $values = array( 'id'=>$id,"value"=>$val, "date"=>date('Y-m-d'),'serialized'=>0 ) ;
							}
							//MyMVC::_getInstance()->_log(MYMVC_DEBUG,'insert registry',MYMVC_LOG);
							$db->_insert($this->_tableName, $values);	
							//MyMVC::_getInstance()->_log( MYMVC_DEBUG ,'registry: '.$db->_getLastQuery(), MYMVC_LOG );
						} else {
							
						}
         }
         $this->_saved = true ;
      } 
  }
  
  /** purge les données de la registry */
  public function _clear()
  {
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::_clear()',MYMVC_LOG);
  	$this->_data = array(); // purge des données en mémoire, la fonctio _save() appliquera en base les suppressions
  	$this->_saved = false;
  }
  
  
	/**
	 *  get a param value
	 * @param  string $pParam
	 * 
	 * @return  null or param value if found
	*/
	public function __get($pParam)
	{
    $pParam = $this->_mymvc_registry_prefix.$pParam; // ajout d'un prefix, pour le multi-user
		if(is_array($this->_data) and array_key_exists( $pParam , $this->_data ) )
		{
			return $this->_data["$pParam"] ;
		} else {
			return null ;
		}
	}
	
	public function _toArray()
	{
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::_toArray()',MYMVC_LOG);
      if( $this->_mymvc_registry_prefix != '' ) {
         $registry = array();
         $l = strlen($this->_mymvc_registry_prefix);
         foreach($this->_data as $ref=>$val) // il faut supprimer le prefix des refs
         {
            $code = substr($ref,$l);
            $registry[] = array($code=>$val);
         }
      } else {
         $registry = $this->_data; // pas de modif si aucun prefix
      }
		return $registry;
	}
	
	/*
	*  set a param 
	*  
	* @param string  $pParam 
	* @param string/array  $pVal
	* 
	* @return true
	* 
	*/
	public function __set($pParam , $pVal)
	{
    $pParam = $this->_mymvc_registry_prefix.$pParam;
		$this->_data[$pParam] = $pVal;
		$this->_saved = false ;
		MyMVC::_getInstance()->_log(MYMVC_INFO,"Maj registry : $pParam=$pVal",MYMVC_LOG);
		//echo ("$pParam => $pVal");
		return true ;
	}
	
	/**
	 * remove a param
	 * 
	 * @param string  $pParam
	 * 
	 * @return bool
	 * 
	 * TODO: check if param exist
	*/
	public function _remove($pParam)
	{
		MyMVC::_getInstance()->_log(MYMVC_DEBUG,'MyMVC_Registry::_remove()',MYMVC_LOG);
		unset($this->_data["$pParam"]) ;
		$this->_saved = false ;
	}
	
	/** 
	 * functions to implements arrayAccess to class 
	*/	
	public function offsetExists($pOffset)
	{
	}
	
	public function offsetGet($pOffset)
	{
		return $this->$pOffset;
	}
	
	public function offsetSet($pOffset,$pVal)
	{
		$this->set($pOffset,$pVal);
		$this->_saved = false ;
	}
	
	public function offsetUnset($pOffset)
	{
		unset($this->_data["$pOffset"]);
		$this->_saved = false ;
	}
	
}

