/*
Copyright (C) 2010 - 2014 Auguste  <postmaster@pajagus.fr>
    
This file is part of MyMVC PHP Framework.

MyMVC PHP Framework is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MyMVC PHP Framework is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MyMVC PHP Framework.  If not, see <http://www.gnu.org/licenses/> 

 * 
 * Class JS pour la création de formulaire multi-etapes
 * chaque étape (step) est composé d'un écran de contenu (texte et/ou input, checkbox etc)
 * 2 boutons  Suivant/Précédent pour naviguer
 * 1 bouton Valider sur la derniere étape
 * 
 *  le wizard est construit dans une page HTML unique avec pour chaque étape une div qui doit avoir un ID spécifique :
 *  
 *  <div id="wizardStep_1" class="wizardStep"> ... </div>  pour l'étape 1
 *   
 *  Vairable à définir dans le <form id="wizard">
 *   
 *  URL d'envoie du formulaire, appel Ajax   <input type="hidden" name="wzUrl" id="wzUrl" value="ajax/archivage">
 *   
 *  Fonction JS de callBack, traitement du retour       <input type="hidden" name="wzCallBack" id="wzCallBack" value="wzArchiver.archivageResult">   	
 *  
 *  Indique le nombre d'étape afin d'identifier la dernière étape notamment  <input type="hidden" name="wzSteps" id="wzSteps" value="2">    								
 *  
 *  Fonction js qui est appelée avec l'affichage de chaque étape et avan le post final  <input type="hidden" name="wzStepPreFunc" id="wzStepPreFunc" value="">   						
 *  
 *  Message de confirmation affiché avec post final  <input type="hidden" name="wzConfirmMessage" id="wzConfirmMessage" value="Confirmer l'archivage des opérations ?">  
 *     
 *  Le message de confirmation et la fonction wsStepPreFunc sont facultatif
 *  
 *  Une div pour les boutons doit etre créée :  <div id="wzBouton"> ..</div>
 *  Les boutons utilisent 3 classes CSS :  wzBtNext, wzBtPrevious et wzBtValid 
 *   
 *  Pour initialiser le wizard il faut ajouter le code Js suivant dans la page HTML
 *  
 *  $(document).ready( function() {
 *   	MyMVC_Wizard.init('wzArchiver');  // wzArchiver =  id du formulaire à transmettre
 *  });
 *   
 *   
 *  25/01/10 - creation
 *  30/01/10 - add wzUrl and wzCallBack params
 *  31/01/10 - gotoStep()
 *  22/02/10 - add wzCurrentStep when call step func
 *  26/01/11 - ajout variable wzStepPostFunc et wzStepPreFunc 
 *  09/03/11 - ajout confirmation avant post du form  utilisation de wzConfirmMessage dans le formulaire
 *  1.0.2
 *  26/06/14 - Intégration des variables globales dans l'objet
 *  
 *  
 *  TODO :
 *  
 *  Ajouter la possibilité de 'passer' un step, bloquer le 3 pour passer du 2 ou 4 par exemple
 *  Affichage des boutons en JS directement
 *   
 *   
*/

MYMVC_JSWIZARD_VERSION = "1.0.2";

var MyMVC_Wizard = { 
	
	currentStep: 1,
	nbStep: 0,
	funcwz: '',	
	
	init: function(name) {
			// hide all steps
			$(".wizardStep").hide();
			
			$("#btNext").click( MyMVC_Wizard.nextStep );
			$("#btPrevious").click( MyMVC_Wizard.previousStep );
			$("#btEnd").click( function () { 
				MyMVC_Wizard.save(name);
				});
			$("#btEnd").hide();
			// init vars
			MyMVC_Wizard.nbStep = $("#wzSteps").val();
			MyMVC_Wizard.funcwz = $("#MyMVC_Wizard.funcwz").val();
			if( MyMVC_Wizard.nbStep > 0 )
			{
				MyMVC_Wizard.currentStep=1;
				MyMVC_Wizard.updateStep(); // load page 1
				//console.log(wzNbPage);
			} else {
				$("#infoEtape").html('Error MyMVC_Wizard.init()');
				$("#btNext").hide();
				$("#btPrevious").hide();
				$("#btEnd").hide();
			}
		},
	
	updateStep: function() {
		// appel func avant passage au step suivant
		MyMVC_Wizard.preFunc(MyMVC_Wizard.currentStep);
		
		$("#wizardStep_"+MyMVC_Wizard.currentStep).slideDown("fast"); //, { direction: 'bottom' }, 1000);
		if(MyMVC_Wizard.currentStep == MyMVC_Wizard.nbStep) // last page => no next, display end bt
		{
			$("#btNext").hide();
			$("#btEnd").show();
		} else if( MyMVC_Wizard.currentStep < MyMVC_Wizard.nbStep )  // more next pages
			{
				$("#btNext").show();
				$("#btEnd").hide();
			}
			
		if(MyMVC_Wizard.currentStep == 1)   // first page => no previoous
		{
			$("#btPrevious").hide();
		} else if( MyMVC_Wizard.currentStep > 1 )
			{
				$("#btPrevious").show();
			}
		$("#infoEtape").html("Etape "+ MyMVC_Wizard.currentStep + " sur " + MyMVC_Wizard.nbStep);
		
		// appel func apres passage au step suivant
		MyMVC_Wizard.postFunc(MyMVC_Wizard.currentStep);
	},
	
	nextStep: function() {
		$("#wizardStep_"+MyMVC_Wizard.currentStep).slideUp("fast");//, { direction: 'top' }, 1000);
		if( MyMVC_Wizard.currentStep < MyMVC_Wizard.nbStep )
		{
			MyMVC_Wizard.currentStep++;
			MyMVC_Wizard.updateStep();
		}
	},
	
	previousStep: function() {
		$("#wizardStep_"+MyMVC_Wizard.currentStep).slideUp("fast");//, { direction: 'top' }, 1000);
		if(MyMVC_Wizard.currentStep > 1)
		{
			MyMVC_Wizard.currentStep--;
			MyMVC_Wizard.updateStep();
		}
	},
	
	save: function(name)
	{
		var confirmOk = '';
		MyMVC_Wizard.preFunc(MyMVC_Wizard.currentStep);
		// affiche confirmation
		if( $("#wzConfirmMessage").length > 0  )
		{
			confirmOk = confirm($("#wzConfirmMessage").val());
		} else {
			confirmOk = true ;
		}
		// envoie ajax si ok
		if( confirmOk ) {
			//eval( f +'()' );   ???
			var wzUrl = $("#wzUrl").val();
			$.post( BASE_URL+wzUrl , $("#"+name ).serialize() , eval($("#wzCallBack").val()) ) ;
		}
	},
	
	gotoStep: function(step)
	{
		$("#wizardStep_"+MyMVC_Wizard.currentStep).hide('fast');
		MyMVC_Wizard.currentStep = step;
		MyMVC_Wizard.updateStep();
	},
	
	preFunc: function(step) {
		// appel func avant passage au step suivant
		if( $("#wzStepPreFunc").length > 0 ) // & $("#MyMVC_Wizard.funcwz").val() != '' )
		{
			var f = $("#wzStepPreFunc").val();
			eval( f +'('+step+')' );
		}
		//console.log(MyMVC_Wizard.currentStep);
	},
	
	postFunc: function(step) {
		// appel func apres passage au step suivant
		if( $("#wzStepPostFunc").length > 0 ) // & $("#MyMVC_Wizard.funcwz").val() != '' )
		{
			var f = $("#wzStepPostFunc").val();
			eval( f +'('+step+')' );
		}
		//console.log(wzCurrentStep);
	}
		

}